from box import BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import ZDXIterator, zdx_params


class UsersAPI(APIEndpoint):
    @zdx_params
    def list_users(self, **kwargs) -> BoxList:
        """
        Returns a list of all active users configured within the ZDX tenant.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.

        Returns:
            :obj:`BoxList`: The list of users in ZDX.

        Examples:
            List all users in ZDX for the past 2 hours:

            >>> for user in zdx.users.list_users():
            ...     print(user)

        """
        return BoxList(ZDXIterator(self._api, "users", **kwargs))

    @zdx_params
    def get_user(self, user_id: str, **kwargs):
        """
        Returns information on the specified user configured within the ZDX tenant.

        Args:
            user_id (str): The unique ID for the ZDX user.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.

        Returns:
            :obj:`Box`: The user information.

        Examples:
            Return information on the user with the ID of 999999999:

            >>> zia.users.get_user(user_id='999999999')

        """
        return self._get(f"users/{user_id}")



import os

from box import Box
from restfly import APISession

from pyzscaler import __version__

from .admin import ZCONAdminAPI
from .config import ZCONConfigAPI
from .connectors import ZCONConnectorsAPI
from .locations import ZCONLocationsAPI
from .session import ZCONSessionAPI


class ZCON(APISession):
    """
    A Controller to access Endpoints in the Zscaler Cloud and Branch Connector API.

    The ZCON object stores the session token and simplifies access to CRUD options within the ZCON Portal.

    """

    _vendor = "Zscaler"
    _product = "Zscaler Cloud and Branch Connector"
    _backoff = 3
    _build = __version__
    _box = True
    _box_attrs = {"camel_killer_box": True}
    _env_base = "ZCON"
    _url = "https://connector.zscaler.net/api/v1"
    _env_cloud = "zscaler"

    def __init__(self, **kw):
        self._api_key = kw.get("api_key", os.getenv(f"{self._env_base}_API_KEY"))
        self._username = kw.get("username", os.getenv(f"{self._env_base}_USERNAME"))
        self._password = kw.get("password", os.getenv(f"{self._env_base}_PASSWORD"))
        self.env_cloud = kw.get("cloud", os.getenv(f"{self._env_base}_CLOUD"))
        self._url = (
            kw.get("override_url", os.getenv(f"{self._env_base}_OVERRIDE_URL"))
            or f"https://connector.{self.env_cloud}.net/api/v1"
        )
        self.conv_box = True
        super(ZCON, self).__init__(**kw)

    def _build_session(self, **kwargs) -> Box:
        """
        Build the APISession object.

        This method is called automatically when instantiating the ZCON object.

        Returns:
            Box: The Box object representing the ZCON API.

        """
        super(ZCON, self)._build_session(**kwargs)
        return self.session.create(api_key=self._api_key, username=self._username, password=self._password)

    def _deauthenticate(self):
        """
        End the authentication session.

        Returns:
            Box: The Box object representing the ZCON API.

        """
        return self.session.delete()

    @property
    def admin(self) -> ZCONAdminAPI:
        """
        The interface object for the :ref:`ZCON Admin interface <zcon-admin>`.

        Returns:
            ZCONAdminAPI: The AdminAPI object.

        """
        return ZCONAdminAPI(self)

    @property
    def connectors(self) -> ZCONConnectorsAPI:
        """
        The interface object for the :ref:`ZCON Connectors interface <zcon-connectors>`.

        Returns:
            ZCONConnectorsAPI: The ConnectorsAPI object.

        """
        return ZCONConnectorsAPI(self)

    @property
    def config(self) -> ZCONConfigAPI:
        """
        The interface object for the :ref:`ZCON Config interface <zcon-config>`.

        Returns:
            ZCONConfigAPI: The ConfigAPI object.

        """
        return ZCONConfigAPI(self)

    @property
    def locations(self) -> ZCONLocationsAPI:
        """
        The interface object for the :ref:`ZCON Locations interface <zcon-locations>`.

        Returns:
            ZCONLocationsAPI: The LocationsAPI object.

        """
        return ZCONLocationsAPI(self)

    @property
    def session(self) -> ZCONSessionAPI:
        """
        The interface object for the :ref:`ZCON Authentication interface <zcon-session>`.

        Returns:
            ZCONSessionAPI: The SessionAPI object.

        """
        return ZCONSessionAPI(self)



import time
from hashlib import sha256

from box import Box
from restfly.endpoint import APIEndpoint


class SessionAPI(APIEndpoint):
    def create_token(self, client_id: str, client_secret: str) -> Box:
        """
        Creates a ZDX authentication token.

        Args:
            client_id (str): The ZDX API Key ID.
            client_secret (str): The ZDX API Key Secret.

        Returns:
            :obj:`Box`: The authenticated session information.

        Examples:
            >>> zia.session.create(client_id='999999999',
            ...    client_secret='admin@example.com')

        """
        epoch_time = int(time.time())

        # Zscaler requires the API Secret Key to be appended with the epoch timestamp, separated by a colon. We then
        # need to take the SHA256 hash of this string and pass that as the API Secret Key.
        api_secret_format = f"{client_secret}:{epoch_time}"
        api_secret_hash = sha256(api_secret_format.encode("utf-8")).hexdigest()

        payload = {"key_id": client_id, "key_secret": api_secret_hash, "timestamp": epoch_time}

        return self._post("oauth/token", json=payload)

    def validate_token(self):
        """
        Validates the current ZDX JWT token.

        Returns:
            :obj:`Box`: The validated session information.

        Examples:
            >>> validation = zdx.session.validate()

        """
        return self._get("oauth/validate")

    def get_jwks(self):
        """
        Returns a JSON Web Key Set (JWKS) that contains the public keys that can be used to verify the JWT tokens.

        Returns:
            :obj:`Box`: The JSON Web Key Set (JWKS).

        Examples:
            >>> jwks = zdx.session.get_jwks()

        """
        return self._get("oauth/jwks")



from box import Box
from restfly import APISession
from restfly.endpoint import APIEndpoint


class CloudSandboxAPI(APIEndpoint):
    def __init__(self, api: APISession):
        super().__init__(api)

        self.sandbox_token = api.sandbox_token
        self.env_cloud = api.env_cloud

    def submit_file(self, file: str, force: bool = False) -> Box:
        """
        Submits a file to the ZIA Advanced Cloud Sandbox for analysis.

        Args:
            file (str): The filename that will be submitted for sandbox analysis.
            force (bool): Force ZIA to analyse the file even if it has been submitted previously.

        Returns:
            :obj:`Box`: The Cloud Sandbox submission response information.

        Examples:
            Submit a file in the current directory called malware.exe to the cloud sandbox, forcing analysis.

            >>> zia.sandbox.submit_file('malware.exe', force=True)

        """
        with open(file, "rb") as f:
            data = f.read()

        params = {
            "api_token": self.sandbox_token,
            "force": int(force),  # convert boolean to int for ZIA
        }

        return self._post(f"https://csbapi.{self.env_cloud}.net/zscsb/submit", params=params, data=data)

    def get_quota(self) -> Box:
        """
        Returns the Cloud Sandbox API quota information for the organisation.

        Returns:
            :obj:`Box`: The Cloud Sandbox quota report.

        Examples:
            >>> pprint(zia.sandbox.get_quota())

        """
        return self._get("sandbox/report/quota")[0]

    def get_report(self, md5_hash: str, report_details: str = "summary") -> Box:
        """
        Returns the Cloud Sandbox Report for the provided hash.

        Args:
            md5_hash (str):
                The MD5 hash of the file that was analysed by Cloud Sandbox.
            report_details (str):
                The type of report. Accepted values are 'full' or 'summary'. Defaults to 'summary'.

        Returns:
            :obj:`Box`: The cloud sandbox report.

        Examples:
            Get a summary report:

            >>> zia.sandbox.get_report('8350dED6D39DF158E51D6CFBE36FB012')

            Get a full report:

            >>> zia.sandbox.get_report('8350dED6D39DF158E51D6CFBE36FB012', 'full')

        """

        return self._get(f"sandbox/report/{md5_hash}?details={report_details}")



import os

from box import Box
from restfly.session import APISession

from pyzscaler import __version__

from .devices import DevicesAPI
from .secrets import SecretsAPI
from .session import AuthenticatedSessionAPI


class ZCC(APISession):
    """
    A Controller to access Endpoints in the Zscaler Mobile Admin Portal API.

    The ZCC object stores the session token and simplifies access to CRUD options within the ZCC Portal.

    Attributes:
        client_id (str): The ZCC Client ID generated from the ZCC Portal.
        client_secret (str): The ZCC Client Secret generated from the ZCC Portal.
        cloud (str): The Zscaler cloud for your tenancy, accepted values are:

            * ``zscaler``
            * ``zscalerone``
            * ``zscalertwo``
            * ``zscalerthree``
            * ``zscloud``
            * ``zscalerbeta``
        override_url (str):
            If supplied, this attribute can be used to override the production URL that is derived
            from supplying the `cloud` attribute. Use this attribute if you have a non-standard tenant URL
            (e.g. internal test instance etc). When using this attribute, there is no need to supply the `cloud`
            attribute. The override URL will be prepended to the API endpoint suffixes. The protocol must be included
            i.e. http:// or https://.

    """

    _vendor = "Zscaler"
    _product = "Zscaler Mobile Admin Portal"
    _backoff = 3
    _build = __version__
    _box = True
    _box_attrs = {"camel_killer_box": True}
    _env_base = "ZCC"
    _env_cloud = "zscaler"
    _url = "https://api-mobile.zscaler.net/papi"

    def __init__(self, **kw):
        self._client_id = kw.get("client_id", os.getenv(f"{self._env_base}_CLIENT_ID"))
        self._client_secret = kw.get("client_secret", os.getenv(f"{self._env_base}_CLIENT_SECRET"))
        self._env_cloud = kw.get("cloud", os.getenv(f"{self._env_base}_CLOUD"))
        self._url = (
            kw.get("override_url", os.getenv(f"{self._env_base}_OVERRIDE_URL"))
            or f"https://api-mobile.{self._env_cloud}.net/papi"
        )
        self.conv_box = True
        super(ZCC, self).__init__(**kw)

    def _build_session(self, **kwargs) -> Box:
        """Creates a ZCC API session."""
        super(ZCC, self)._build_session(**kwargs)
        self._auth_token = self.session.create_token(client_id=self._client_id, client_secret=self._client_secret)
        return self._session.headers.update({"auth-token": f"{self._auth_token}"})

    @property
    def devices(self):
        """The interface object for the :ref:`ZCC Devices interface <zcc-devices>`."""
        return DevicesAPI(self)

    @property
    def secrets(self):
        """The interface object for the :ref:`ZCC Secrets interface <zcc-secrets>`."""
        return SecretsAPI(self)

    @property
    def session(self):
        """The interface object for the :ref:`ZCC Authenticated Session interface <zcc-session>`."""
        return AuthenticatedSessionAPI(self)



from box import BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import zdx_params


class AdminAPI(APIEndpoint):
    @zdx_params
    def list_departments(self, **kwargs) -> BoxList:
        """
        Returns a list of departments that are configured within ZDX.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            search (str): The search string to filter by name or department ID.

        Returns:
            :obj:`BoxList`: The list of departments in ZDX.

        Examples:
            List all departments in ZDX for the past 2 hours:

            >>> for department in zdx.admin.list_departments():
            ...     print(department)

        """

        return self._get("administration/departments", params=kwargs)

    @zdx_params
    def list_locations(self, **kwargs) -> BoxList:
        """
        Returns a list of locations that are configured within ZDX.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            search (str): The search string to filter by name or location ID.

        Returns:
            :obj:`BoxList`: The list of locations in ZDX.

        Examples:
            List all locations in ZDX for the past 2 hours:

            >>> for location in zdx.admin.list_locations():
            ...     print(location)

        """
        return self._get("administration/locations", params=kwargs)

    @zdx_params
    def list_geolocations(self, **kwargs) -> BoxList:
        """
        Returns a list of all active geolocations configured within the ZDX tenant.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            parent_geo_id (str): The unique ID for the parent geolocation.
            search (str): The search string to filter by name.

        Returns:
            :obj:`BoxList`: The list of geolocations in ZDX.

        Examples:
            List all geolocations in ZDX for the past 2 hours:

            >>> for geolocation in zdx.admin.list_geolocations():
            ...     print(geolocation)

        """
        return self._get("active_geo", params=kwargs)



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, convert_keys, snake_to_camel


class PolicySetsAPI(APIEndpoint):
    POLICY_MAP = {
        "access": "ACCESS_POLICY",
        "timeout": "TIMEOUT_POLICY",
        "client_forwarding": "CLIENT_FORWARDING_POLICY",
        "siem": "SIEM_POLICY",
    }

    def _create_conditions(self, conditions: list) -> list:
        """
        Creates a list template for feeding conditions into the ZPA Policies API when adding or updating a policy.

        Args:
            conditions (list): List of condition tuples or lists (containing more complex logic statements).

        Returns:
            list: The conditions template.

        """
        final_template = []
        for condition in conditions:
            template, operator = self._parse_condition(condition)
            expression = {"operands": template}
            if operator:
                expression["operator"] = operator.upper()
            final_template.append(expression)
        return final_template

    def _parse_condition(self, condition):
        """
        Transforms a single statement with operand into a format for a condition template.

        Args:
            condition: A single list of condition statements or a tuple.

        Returns:
            tuple: A tuple containing the formatted condition template (list) and the operator (str, "AND" or "OR")
            or None.

        """
        # If this is a tuple condition on its own, process it now
        if isinstance(condition, tuple) and len(condition) == 3:
            return [self._format_condition_tuple(condition)], None

        # Otherwise we'd expect a list of tuples and an optional operator at the end
        template = []
        operator = None
        for parameter in condition:
            if isinstance(parameter, str) and parameter.upper() in ["OR", "AND"]:
                operator = parameter
            elif isinstance(parameter, tuple) and len(parameter) == 3:
                template.append(self._format_condition_tuple(parameter))
        return template, operator

    @staticmethod
    def _format_condition_tuple(condition: tuple):
        """
        Formats a simple tuple condition.

        Args:
            condition (tuple): A condition tuple (objectType, lhs, rhs).

        Returns:
            dict: Formatted dict structure for ZIA Policies API.

        """
        return {
            "objectType": condition[0].upper(),
            "lhs": condition[1],
            "rhs": condition[2],
        }

    def get_policy(self, policy_type: str) -> Box:
        """
        Returns the policy and rule sets for the given policy type.

        Args:
            policy_type (str): The type of policy to be returned. Accepted values are:

                 |  ``access`` - returns the Access Policy
                 |  ``timeout`` - returns the Timeout Policy
                 |  ``client_forwarding`` - returns the Client Forwarding Policy
                 |  ``siem`` - returns the SIEM Policy

        Returns:
            :obj:`Box`: The resource record of the specified policy type.

        Examples:
            Request the specified Policy.

            >>> pprint(zpa.policies.get_policy('access'))

        """
        # Map the simplified policy_type name to the name expected by the Zscaler API
        mapped_policy_type = self.POLICY_MAP.get(policy_type, None)

        # If the user provided an incorrect name, raise an error
        if not mapped_policy_type:
            raise ValueError(
                f"Incorrect policy type provided: {policy_type}\n "
                f"Policy type must be 'access', 'timeout', 'client_forwarding' or 'siem'."
            )

        return self._get(f"policySet/policyType/{mapped_policy_type}")

    def get_rule(self, policy_type: str, rule_id: str) -> Box:
        """
        Returns the specified policy rule.

        Args:
            policy_type (str): The type of policy to be returned. Accepted values are:

                 |  ``access``
                 |  ``timeout``
                 |  ``client_forwarding``
                 |  ``siem``
            rule_id (str): The unique identifier for the policy rule.

        Returns:
            :obj:`Box`: The resource record for the requested rule.

        Examples:
            >>> policy_rule = zpa.policies.get_rule(policy_id='99999',
            ...    rule_id='88888')

        """
        # Get the policy id for the supplied policy_type
        policy_id = self.get_policy(policy_type).id

        return self._get(f"policySet/{policy_id}/rule/{rule_id}")

    def list_rules(self, policy_type: str, **kwargs) -> BoxList:
        """
        Returns policy rules for a given policy type.

        Args:
            policy_type (str):
                The policy type. Accepted values are:

                |  ``access`` - returns Access Policy rules
                |  ``timeout`` - returns Timeout Policy rules
                |  ``client_forwarding`` - returns Client Forwarding Policy rules

        Returns:
            :obj:`list`: A list of all policy rules that match the requested type.

        Examples:
            >>> for policy in zpa.policies.list_type('type')
            ...    pprint(policy)

        """

        # Map the simplified policy_type name to the name expected by the Zscaler API
        mapped_policy_type = self.POLICY_MAP.get(policy_type, None)

        # If the user provided an incorrect name, raise an error
        if not mapped_policy_type:
            raise ValueError(
                f"Incorrect policy type provided: {policy_type}\n "
                f"Policy type must be 'access', 'timeout', 'client_forwarding' or 'siem'."
            )

        return BoxList(Iterator(self._api, f"policySet/rules/policyType/{mapped_policy_type}", **kwargs))

    def delete_rule(self, policy_type: str, rule_id: str) -> int:
        """
        Deletes the specified policy rule.

        Args:
            policy_type (str):
                The type of policy the rule belongs to. Accepted values are:

                 |  ``access``
                 |  ``timeout``
                 |  ``client_forwarding``
                 |  ``siem``
            rule_id (str):
                The unique identifier for the policy rule.

        Returns:
            :obj:`int`: The response code for the operation.

        Examples:
            >>> zpa.policies.delete_rule(policy_id='99999',
            ...    rule_id='88888')

        """

        # Get policy id for specified policy type
        policy_id = self.get_policy(policy_type).id

        return self._delete(f"policySet/{policy_id}/rule/{rule_id}").status_code

    def add_access_rule(self, name: str, action: str, **kwargs) -> Box:
        """
        Add a new Access Policy rule.

        See the `ZPA Access Policy API reference <https://help.zscaler.com/zpa/access-policy-use-cases>`_
        for further detail on optional keyword parameter structures.

        Args:
            name (str):
                The name of the new rule.
            action (str):
                The action for the policy. Accepted values are:

                |  ``allow``
                |  ``deny``
            **kwargs:
                Optional keyword args.

        Keyword Args:
            conditions (list):
                A list of conditional rule tuples. Tuples must follow the convention: `Object Type`, `LHS value`,
                `RHS value`. If you are adding multiple values for the same object type then you will need
                a new entry for each value.
                E.g.

                .. code-block:: python

                    [('app', 'id', '99999'),
                    ('app', 'id', '88888'),
                    ('app_group', 'id', '77777),
                    ('client_type', 'zpn_client_type_exporter', 'zpn_client_type_zapp'),
                    ('trusted_network', 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx', True)]
            custom_msg (str):
                A custom message.
            description (str):
                A description for the rule.

        Returns:
            :obj:`Box`: The resource record of the newly created access policy rule.

        """

        # Initialise the payload
        payload = {
            "name": name,
            "action": action.upper(),
            "conditions": self._create_conditions(kwargs.pop("conditions", [])),
        }

        # Get the policy id of the provided policy type for the URL.
        policy_id = self.get_policy("access").id

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post(f"policySet/{policy_id}/rule", json=payload)

    def add_timeout_rule(self, name: str, **kwargs) -> Box:
        """
        Add a new Timeout Policy rule.

        See the `ZPA Timeout Policy API reference <https://help.zscaler.com/zpa/timeout-policy-use-cases>`_
        for further detail on optional keyword parameter structures.

        Args:
            name (str):
                The name of the new rule.
            **kwargs:
                Optional parameters.

        Keyword Args:
            conditions (list):
                A list of conditional rule tuples. Tuples must follow the convention: `Object Type`, `LHS value`,
                `RHS value`. If you are adding multiple values for the same object type then you will need
                a new entry for each value.
                E.g.

                .. code-block:: python

                    [('app', 'id', '926196382959075416'),
                    ('app', 'id', '926196382959075417'),
                    ('app_group', 'id', '926196382959075332),
                    ('client_type', 'zpn_client_type_exporter', 'zpn_client_type_zapp'),
                    ('trusted_network', 'b15e4cad-fa6e-8182-9fc3-8125ee6a65e1', True)]
            custom_msg (str):
                A custom message.
            description (str):
                A description for the rule.
            re_auth_idle_timeout (int):
                The re-authentication idle timeout value in seconds.
            re_auth_timeout (int):
                The re-authentication timeout value in seconds.

        Returns:
            :obj:`Box`: The resource record of the newly created Timeout Policy rule.

        """

        # Initialise the payload
        payload = {"name": name, "action": "RE_AUTH", "conditions": self._create_conditions(kwargs.pop("conditions", []))}

        # Get the policy id of the provided policy type for the URL.
        _policy_id = self.get_policy("timeout").id

        # Use specified timeouts or default to UI values
        payload["reauthTimeout"] = kwargs.get("re_auth_timeout", 172800)
        payload["reauthIdleTimeout"] = kwargs.get("re_auth_idle_timeout", 600)

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post(f"policySet/{_policy_id}/rule", json=payload)

    def add_client_forwarding_rule(self, name: str, action: str, **kwargs) -> Box:
        """
        Add a new Client Forwarding Policy rule.

        See the
        `ZPA Client Forwarding Policy API reference <https://help.zscaler.com/zpa/client-forwarding-policy-use-cases>`_
        for further detail on optional keyword parameter structures.

        Args:
            name (str):
                The name of the new rule.
            action (str):
                The action for the policy. Accepted values are:

                |  ``intercept``
                |  ``intercept_accessible``
                |  ``bypass``
            **kwargs:
                Optional keyword args.

        Keyword Args:
            conditions (list):
                A list of conditional rule tuples. Tuples must follow the convention: `Object Type`, `LHS value`,
                `RHS value`. If you are adding multiple values for the same object type then you will need
                a new entry for each value.
                E.g.

                .. code-block:: python

                    [('app', 'id', '926196382959075416'),
                    ('app', 'id', '926196382959075417'),
                    ('app_group', 'id', '926196382959075332),
                    ('client_type', 'zpn_client_type_exporter', 'zpn_client_type_zapp'),
                    ('trusted_network', 'b15e4cad-fa6e-8182-9fc3-8125ee6a65e1', True)]
            custom_msg (str):
                A custom message.
            description (str):
                A description for the rule.

        Returns:
            :obj:`Box`: The resource record of the newly created Client Forwarding Policy rule.

        """

        # Initialise the payload
        payload = {"name": name, "action": action.upper(), "conditions": self._create_conditions(kwargs.pop("conditions", []))}

        # Get the policy id of the provided policy type for the URL.
        policy_id = self.get_policy("client_forwarding").id

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post(f"policySet/{policy_id}/rule", json=payload)

    def update_rule(self, policy_type: str, rule_id: str, **kwargs) -> Box:
        """
        Update an existing policy rule.

        Ensure you are using the correct arguments for the policy type that you want to update.

        Args:
            policy_type (str):
                The policy type. Accepted values are:

                 |  ``access``
                 |  ``timeout``
                 |  ``client_forwarding``
            rule_id (str):
                The unique identifier for the rule to be updated.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            action (str):
                The action for the policy. Accepted values are:

                |  ``allow``
                |  ``deny``
                |  ``intercept``
                |  ``intercept_accessible``
                |  ``bypass``
            conditions (list):
                A list of conditional rule tuples. Tuples must follow the convention: `Object Type`, `LHS value`,
                `RHS value`. If you are adding multiple values for the same object type then you will need
                a new entry for each value.
                E.g.

                .. code-block:: python

                    [('app', 'id', '926196382959075416'),
                    ('app', 'id', '926196382959075417'),
                    ('app_group', 'id', '926196382959075332),
                    ('client_type', 'zpn_client_type_exporter', 'zpn_client_type_zapp'),
                    ('trusted_network', 'b15e4cad-fa6e-8182-9fc3-8125ee6a65e1', True)]
            custom_msg (str):
                A custom message.
            description (str):
                A description for the rule.
            re_auth_idle_timeout (int):
                The re-authentication idle timeout value in seconds.
            re_auth_timeout (int):
                The re-authentication timeout value in seconds.

        Returns:
            :obj:`Box`: The updated policy-rule resource record.

        Examples:
            Updates the name only for an Access Policy rule:

            >>> zpa.policies.update_rule('access', '99999', name='new_rule_name')

            Updates the action only for a Client Forwarding Policy rule:

            >>> zpa.policies.update_rule('client_forwarding', '888888', action='BYPASS')

        """
        # Get policy id for specified policy type
        policy_id = self.get_policy(policy_type).id

        payload = convert_keys(self.get_rule(policy_type, rule_id))

        # Add optional parameters to payload
        for key, value in kwargs.items():
            if key == "conditions":
                payload["conditions"] = self._create_conditions(value)
            else:
                payload[snake_to_camel(key)] = value

        resp = self._put(f"policySet/{policy_id}/rule/{rule_id}", json=payload, box=False).status_code

        if resp == 204:
            return self.get_rule(policy_type, rule_id)

    def reorder_rule(self, policy_type: str, rule_id: str, order: str) -> Box:
        """
        Change the order of an existing policy rule.

        Args:
            rule_id (str):
                The unique id of the rule that will be reordered.
            order (str):
                The new order for the rule.
            policy_type (str):
                The policy type. Accepted values are:

                 |  ``access``
                 |  ``timeout``
                 |  ``client_forwarding``

        Returns:
             :obj:`Box`: The updated policy rule resource record.

        Examples:
            Updates the order for an existing policy rule:

            >>> zpa.policies.reorder_rule(policy_type='access',
            ...    rule_id='88888',
            ...    order='2')

        """
        # Get policy id for specified policy type
        policy_id = self.get_policy(policy_type).id

        resp = self._put(f"policySet/{policy_id}/rule/{rule_id}/reorder/{order}").status_code

        if resp == 204:
            return self.get_rule(policy_type, rule_id)



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import (
    Iterator,
    add_id_groups,
    pick_version_profile,
    snake_to_camel,
)


class ServiceEdgesAPI(APIEndpoint):
    # Parameter names that will be reformatted to be compatible with ZPAs API
    reformat_params = [
        ("service_edge_ids", "serviceEdges"),
        ("trusted_network_ids", "trustedNetworks"),
    ]

    def list_service_edges(self, **kwargs) -> BoxList:
        """
        Returns information on all configured ZPA Service Edges.

        Args:
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to match against a department's name or comments attributes.

        Returns:
            :obj:`BoxList`: List containing information on all configured ZPA Service Edges.

        Examples:
            >>> for service_edge in zpa.service_edges.list_service_edges():
            ...    print(service_edge)

        """
        return BoxList(Iterator(self._api, "serviceEdge", **kwargs))

    def get_service_edge(self, service_edge_id: str) -> Box:
        """
        Returns information on the specified Service Edge.

        Args:
            service_edge_id (str): The unique id of the ZPA Service Edge.

        Returns:
            :obj:`Box`: The Service Edge resource record.

        Examples:
            >>> service_edge = zpa.service_edges.get_service_edge('999999')

        """
        return self._get(f"serviceEdge/{service_edge_id}")

    def update_service_edge(self, service_edge_id: str, **kwargs) -> Box:
        """
        Updates the specified ZPA Service Edge.

        Args:
            service_edge_id (str): The unique id of the Service Edge that will be updated in ZPA.
            **kwargs: Optional keyword args.

        Keyword Args:
            **description (str): Additional information about the Service Edge.
            **enabled (bool): Enable the Service Edge. Defaults to ``True``.
            **name (str): The name of the Service Edge in ZPA.

        Returns:
            :obj:`Box`: The updated Service Edge resource record.

        Examples:
            >>> updated_service_edge = zpa.service_edge.update_service_edge('99999',
            ...    description="Updated Description",
            ...    name="Updated Name")

        """

        # Set payload to equal existing record
        payload = {snake_to_camel(k): v for k, v in self.get_service_edge(service_edge_id).items()}

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"serviceEdge/{service_edge_id}", json=payload).status_code

        if resp == 204:
            return self.get_service_edge(service_edge_id)

    def delete_service_edge(self, service_edge_id: str) -> int:
        """
        Deletes the specified Service Edge from ZPA.

        Args:
            service_edge_id (str): The unique id of the ZPA Service Edge that will be deleted.

        Returns:
            :obj:`int`: The status code of the operation.

        Examples:
            >>> zpa.service_edges.delete_service_edge('99999')

        """
        return self._delete(f"serviceEdge/{service_edge_id}").status_code

    def bulk_delete_service_edges(self, service_edge_ids: list) -> int:
        """
        Bulk deletes the specified Service Edges from ZPA.

        Args:
            service_edge_ids (list): A list of Service Edge ids that will be deleted from ZPA.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            >>> zpa.service_edges.bulk_delete_service_edges(['99999', '88888'])

        """
        payload = {
            "ids": service_edge_ids,
        }

        return self._post("serviceEdge/bulkDelete", json=payload).status_code

    def list_service_edge_groups(self, **kwargs) -> BoxList:
        """
        Returns information on all configured Service Edge Groups in ZPA.

        Args:
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to match against a department's name or comments attributes.

        Returns:
            :obj:`BoxList`: A list of all ZPA Service Edge Group resource records.

        Examples:
            Print all Service Edge Groups in ZPA.

            >>> for group in zpa.service_edges.list_service_edge_groups():
            ...    print(group)

        """
        return BoxList(Iterator(self._api, "serviceEdgeGroup", **kwargs))

    def get_service_edge_group(self, group_id: str) -> Box:
        """
        Returns information on the specified ZPA Service Edge Group.

        Args:
            group_id (str): The unique id of the ZPA Service Edge Group.

        Returns:
            :obj:`Box`: The specified ZPA Service Edge Group resource record.

        Examples:
            >>> group = zpa.service_edges.get_service_edge_group("99999")

        """
        return self._get(f"serviceEdgeGroup/{group_id}")

    def add_service_edge_group(self, name: str, latitude: str, longitude: str, location: str, **kwargs):
        """
        Adds a new Service Edge Group to ZPA.

        Args:
            latitude (str): The latitude representing the physical location of the ZPA Service Edges in this group.
            longitude (str): The longitude representing the physical location of the ZPA Service Edges in this group.
            location (str): The name of the physical location of the ZPA Service Edges in this group.
            name (str): The name of the Service Edge Group.
            **kwargs: Optional keyword args.

        Keyword Args:
            **cityCountry (str):
                The City and Country for where the App Connectors are located. Format is:

                ``<City>, <Country Code>`` e.g. ``Sydney, AU``
            **country_code (str):
                The ISO<std> Country Code that represents the country where the App Connectors are located.
            **enabled (bool):
                Is the Service Edge Group enabled? Defaults to ``True``.
            **is_public (bool):
                Is the Service Edge publicly accessible? Defaults to ``False``.
            **override_version_profile (bool):
                Override the local App Connector version according to ``version_profile``. Defaults to ``False``.
            **service_edge_ids (list):
                A list of unique ids of ZPA Service Edges that belong to this Service Edge Group.
            **trusted_network_ids (list):
                A list of unique ids of Trusted Networks that are associated with this Service Edge Group.
            **upgrade_day (str):
                The day of the week that upgrades will be pushed to the App Connector.
            **upgrade_time_in_secs (str):
                The time of the day that upgrades will be pushed to the App Connector.
            **version_profile (str):
                The version profile to use. This will automatically set ``override_version_profile`` to True.
                Accepted values are:

                ``default``, ``previous_default`` and ``new_release``

        Returns:
            :obj:`Box`: The resource record of the newly created Service Edge Group.

        Examples:
            Add a new Service Edge Group for Service Edges in Sydney and set the version profile to new releases.

            >>> group = zpa.service_edges.add_service_edge_group(name="My SE Group",
            ...    latitude="33.8688",
            ...    longitude="151.2093",
            ...    location="Sydney",
            ...    version_profile="new_release)

        """
        payload = {
            "name": name,
            "latitude": latitude,
            "longitude": longitude,
            "location": location,
        }

        # Perform formatting on simplified params
        add_id_groups(self.reformat_params, kwargs, payload)
        pick_version_profile(kwargs, payload)

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post("serviceEdgeGroup", json=payload)

    def update_service_edge_group(self, group_id: str, **kwargs) -> Box:
        """
        Updates the specified ZPA Service Edge Group.

        Args:
            group_id (str): The unique id of the ZPA Service Edge Group that will be updated.
            **kwargs: Optional keyword args.

        Keyword Args:
            **cityCountry (str):
                The City and Country for where the App Connectors are located. Format is:

                ``<City>, <Country Code>`` e.g. ``Sydney, AU``
            **country_code (str):
                The ISO<std> Country Code that represents the country where the App Connectors are located.
            **enabled (bool):
                Is the Service Edge Group enabled? Defaults to ``True``.
            **is_public (bool):
                Is the Service Edge publicly accessible? Defaults to ``False``.
            **latitude (str):
                The latitude representing the physical location of the ZPA Service Edges in this group.
            **longitude (str):
                The longitude representing the physical location of the ZPA Service Edges in this group.
            **location (str): T
                he name of the physical location of the ZPA Service Edges in this group.
            **name (str):
                The name of the Service Edge Group.
            **override_version_profile (bool):
                Override the local App Connector version according to ``version_profile``. Defaults to ``False``.
            **service_edge_ids (list):
                A list of unique ids of ZPA Service Edges that belong to this Service Edge Group.
            **trusted_network_ids (list):
                A list of unique ids of Trusted Networks that are associated with this Service Edge Group.
            **upgrade_day (str):
                The day of the week that upgrades will be pushed to the Service Edges in this group.
            **upgrade_time_in_secs (str):
                The time of the day that upgrades will be pushed to the Service Edges in this group.
            **version_profile (str):
                The version profile to use. This will automatically set ``override_version_profile`` to True.
                Accepted values are:

                ``default``, ``previous_default`` and ``new_release``

        Returns:
            :obj:`Box`: The updated ZPA Service Edge Group resource record.

        Examples:
            Update the name of a Service Edge Group, change the Version Profile to 'default' and the upgrade day to
            Friday.

            >>> group = zpa.service_edges.update_service_edge_group('99999',
            ...    name="Updated Name",
            ...    version_profile="default",
            ...    upgrade_day="friday")

        """
        # Set payload to equal existing record
        payload = {snake_to_camel(k): v for k, v in self.get_service_edge_group(group_id).items()}

        # Perform formatting on simplified params
        add_id_groups(self.reformat_params, kwargs, payload)
        pick_version_profile(kwargs, payload)

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"serviceEdgeGroup/{group_id}", json=payload).status_code

        if resp == 204:
            return self.get_service_edge_group(group_id)

    def delete_service_edge_group(self, service_edge_group_id: str) -> int:
        """
        Deletes the specified Service Edge Group from ZPA.

        Args:
            service_edge_group_id (str): The unique id of the ZPA Service Edge Group.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            >>> zpa.service_edges.delete_service_edge_group("99999")

        """
        return self._delete(f"serviceEdgeGroup/{service_edge_group_id}").status_code



from box import Box
from restfly.endpoint import APIEndpoint


class AuthenticatedSessionAPI(APIEndpoint):
    def create_token(self, client_id: str, client_secret: str) -> Box:
        """
        Creates a ZCC authentication token.

        Args:
            client_id (str): The ZCC Portal Client ID.
            client_secret (str): The ZCC Portal Client Secret.

        Returns:
            :obj:`Box`: The authenticated session information.

        Examples:
            >>> zia.session.create(api_key='999999999',
            ...    username='admin@example.com',
            ...    password='MyInsecurePassword')


        """

        payload = {
            "apiKey": client_id,
            "secretKey": client_secret,
        }

        return self._post("auth/v1/login", json=payload).jwt_token



import functools
import re
import time

from box import Box, BoxList
from restfly import APIIterator


def snake_to_camel(name: str):
    """Converts Python Snake Case to Zscaler's lower camelCase."""
    if "_" not in name:
        return name
    # Edge-cases where camelCase is breaking
    edge_cases = {
        "routable_ip": "routableIP",
        "is_name_l10n_tag": "isNameL10nTag",
        "name_l10n_tag": "nameL10nTag",
        "surrogate_ip": "surrogateIP",
        "surrogate_ip_enforced_for_known_browsers": "surrogateIPEnforcedForKnownBrowsers",
        "ec_vms": "ecVMs",
        "ipv6_enabled": "ipV6Enabled",
        "valid_ssl_certificate": "validSSLCertificate",
    }
    return edge_cases.get(name, name[0].lower() + name.title()[1:].replace("_", ""))


def camel_to_snake(name: str):
    """Converts Zscaler's lower camelCase to Python Snake Case."""
    edge_cases = {
        "routableIP": "routable_ip",
        "isNameL10nTag": "is_name_l10n_tag",
        "nameL10nTag": "name_l10n_tag",
        "surrogateIP": "surrogate_ip",
        "surrogateIPEnforcedForKnownBrowsers": "surrogate_ip_enforced_for_known_browsers",
        "ecVMs": "ec_vms",
        "ipV6Enabled": "ipv6_enabled",
        "validSSLCertificate": "valid_ssl_certificate",
    }
    # Check if name is an edge case
    if name in edge_cases:
        return edge_cases[name]

    name = re.sub("([a-z0-9])([A-Z])", r"\1_\2", name)
    return name.lower()


def chunker(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def convert_keys(data, direction="to_camel"):
    converter = camel_to_snake if direction == "to_snake" else snake_to_camel

    if isinstance(data, (list, BoxList)):
        return [convert_keys(inner_dict, direction=direction) for inner_dict in data]
    elif isinstance(data, (dict, Box)):
        new_dict = {}
        for k in data.keys():
            v = data[k]
            new_key = converter(k)
            new_dict[new_key] = convert_keys(v, direction=direction) if isinstance(v, (dict, list)) else v
        return new_dict
    else:
        return data


def keys_exists(element: dict, *keys):
    """
    Check if *keys (nested) exists in `element` (dict).
    """
    if not isinstance(element, dict):
        raise AttributeError("keys_exists() expects dict as first argument.")
    if not keys:
        raise AttributeError("keys_exists() expects at least two arguments, one given.")

    _element = element
    for key in keys:
        try:
            _element = _element[key]
        except KeyError:
            return False
    return True


# Takes a tuple if id_groups, kwargs and the payload dict; reformat for API call
def add_id_groups(id_groups: list, kwargs: dict, payload: dict):
    for entry in id_groups:
        if kwargs.get(entry[0]):
            payload[entry[1]] = [{"id": param_id} for param_id in kwargs.pop(entry[0])]
    return


def obfuscate_api_key(seed: list):
    now = int(time.time() * 1000)
    n = str(now)[-6:]
    r = str(int(n) >> 1).zfill(6)
    key = "".join(seed[int(str(n)[i])] for i in range(len(str(n))))
    for j in range(len(r)):
        key += seed[int(r[j]) + 2]

    return {"timestamp": now, "key": key}


def pick_version_profile(kwargs: list, payload: list):
    if version_profile := kwargs.pop("version_profile", None):
        payload["overrideVersionProfile"] = True
        if version_profile == "default":
            payload["versionProfileId"] = 0
        elif version_profile == "previous_default":
            payload["versionProfileId"] = 1
        elif version_profile == "new_release":
            payload["versionProfileId"] = 2


class Iterator(APIIterator):
    """Iterator class."""

    page_size = 100

    def __init__(self, api, path: str = "", **kw):
        """Initialize Iterator class."""
        super().__init__(api, **kw)

        self.path = path
        self.max_items = kw.pop("max_items", 0)
        self.max_pages = kw.pop("max_pages", 0)
        self.payload = {}
        if kw:
            self.payload = {snake_to_camel(key): value for key, value in kw.items()}

    def _get_page(self) -> None:
        """Iterator function to get the page."""
        resp = self._api.get(
            self.path,
            params={**self.payload, "page": self.num_pages + 1},
        )
        try:
            # If we are using ZPA then the API will return records under the
            # 'list' key.
            self.page = resp.get("list") or []
        except AttributeError:
            # If the list key doesn't exist then we're likely using ZIA so just
            # return the full response.
            self.page = resp
        finally:
            # If we use the default retry-after logic in Restfly then we are
            # going to keep seeing 429 messages in stdout. ZIA and ZPA have a
            # standard 1 sec rate limit on the API endpoints with pagination so
            # we are going to include it here.
            time.sleep(1)


class ZDXIterator(APIIterator):
    """
    Iterator class for ZDX endpoints.

    """

    def __init__(self, api, endpoint, limit=None, **kwargs):
        super().__init__(api, **kwargs)
        self.endpoint = endpoint
        self.limit = limit
        self.next_offset = None
        self.total = 0

        # Load the first page
        self._get_page()

    def __next__(self):
        try:
            item = super().__next__()
        except StopIteration:
            if self.next_offset is None:
                # There is no next page, so we're done iterating
                raise
            # There is another page, so get it and continue iterating
            self._get_page()
            item = super().__next__()
        return item

    def _get_page(self):
        params = {"limit": self.limit, "offset": self.next_offset} if self.next_offset else {}

        # Request the next page
        response = self._api.get(self.endpoint, params=params)

        # Extract the next offset and the data items from the response
        self.next_offset = response.get("next_offset")
        self.page = response["users"]

        # Update the total number of records
        self.total += len(self.page)

        # Reset page_count for the new page
        self.page_count = 0


# Maps ZCC numeric os_type and registration_type arguments to a human-readable string
zcc_param_map = {
    "os": {
        "ios": 1,
        "android": 2,
        "windows": 3,
        "macos": 4,
        "linux": 5,
    },
    "reg_type": {
        "all": 0,
        "registered": 1,
        "removal_pending": 3,
        "unregistered": 4,
        "removed": 5,
        "quarantined": 6,
    },
}


def calculate_epoch(hours: int):
    current_time = int(time.time())
    past_time = int(current_time - (hours * 3600))
    return current_time, past_time


def zdx_params(func):
    """
    Decorator to add custom parameter functionality for ZDX API calls.

    Args:
        func: The function to decorate.

    Returns:
        The decorated function.

    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        since = kwargs.pop("since", None)
        search = kwargs.pop("search", None)
        location_id = kwargs.pop("location_id", None)
        department_id = kwargs.pop("department_id", None)
        geo_id = kwargs.pop("geo_id", None)

        if since:
            current_time, past_time = calculate_epoch(since)
            kwargs["to"] = current_time
            kwargs["from"] = past_time

        kwargs["q"] = search or kwargs.get("q")
        kwargs["loc"] = location_id or kwargs.get("loc")
        kwargs["dept"] = department_id or kwargs.get("dept")
        kwargs["geo"] = geo_id or kwargs.get("geo")

        return func(self, *args, **kwargs)

    return wrapper



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import (
    Iterator,
    add_id_groups,
    pick_version_profile,
    snake_to_camel,
)


class ConnectorsAPI(APIEndpoint):
    reformat_params = [
        ("connector_ids", "connectors"),
        ("server_group_ids", "serverGroups"),
    ]

    def list_connectors(self, **kwargs) -> BoxList:
        """
        Returns a list of all configured App Connectors.

        Args:
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to match against a department's name or comments attributes.

        Returns:
            :obj:`BoxList`: List containing all configured ZPA App Connectors.

        Examples:
            List all configured App Connectors:

            >>> for connector in zpa.connectors.list_connectors():
            ...    print(connector)

        """
        return BoxList(Iterator(self._api, "connector", **kwargs))

    def get_connector(self, connector_id: str) -> Box:
        """
        Returns information on the specified App Connector.

        Args:
            connector_id (str): The unique id for the ZPA App Connector.

        Returns:
            :obj:`Box`: The specified App Connector resource record.

        Examples:
            >>> app_connector = zpa.connectors.get_connector('99999')

        """
        return self._get(f"connector/{connector_id}")

    def update_connector(self, connector_id: str, **kwargs):
        """
        Updates an existing ZPA App Connector.

        Args:
            connector_id (str): The unique id of the ZPA App Connector.
            **kwargs: Optional keyword args.

        Keyword Args:
            **description (str): Additional information about the App Connector.
            **enabled (bool): True if the App Connector is enabled.
            **name (str): The name of the App Connector.

        Returns:
            :obj:`Box`: The updated App Connector resource record.

        Examples:
            Update an App Connector name and disable it.

            >>> app_connector = zpa.connectors.update_connector('999999',
            ...    name="Updated App Connector Name",
            ...    enabled=False)

        """
        # Set payload to equal existing record
        payload = {snake_to_camel(k): v for k, v in self.get_connector(connector_id).items()}

        # Perform formatting on simplified params
        add_id_groups(self.reformat_params, kwargs, payload)

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"connector/{connector_id}", json=payload).status_code

        if resp == 204:
            return self.get_connector(connector_id)

    def delete_connector(self, connector_id: str) -> int:
        """
        Deletes the specified App Connector from ZPA.

        Args:
            connector_id (str): The unique id for the ZPA App Connector that will be deleted.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            >>> zpa.connectors.delete_connector('999999')

        """
        return self._delete(f"connector/{connector_id}", box=False).status_code

    def bulk_delete_connectors(self, connector_ids: list) -> int:
        """
        Deletes all specified App Connectors from ZPA.

        Args:
            connector_ids (list): The list of unique ids for the ZPA App Connectors that will be deleted.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            >>> zpa.connectors.bulk_delete_connectors(['111111', '222222', '333333'])

        """
        payload = {"ids": connector_ids}
        return self._post("connector/bulkDelete", json=payload, box=False).status_code

    def list_connector_groups(self, **kwargs) -> BoxList:
        """
        Returns a list of all connector groups.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to match against a department's name or comments attributes.

        Returns:
            :obj:`BoxList`: List of all configured connector groups.

        Examples:
            >>> connector_groups = zpa.connectors.list_connector_groups()

        """
        return BoxList(Iterator(self._api, "appConnectorGroup", **kwargs))

    def get_connector_group(self, group_id: str) -> Box:
        """
        Gets information for a specified connector group.

        Args:
            group_id (str):
                The unique identifier for the connector group.

        Returns:
            :obj:`Box`:
                The connector group resource record.

        Examples:
            >>> connector_group = zpa.connectors.get_connector_group('99999')

        """
        return self._get(f"appConnectorGroup/{group_id}")

    def add_connector_group(self, name: str, latitude: int, location: str, longitude: int, **kwargs) -> Box:
        """
        Adds a new ZPA App Connector Group.

        Args:
            name (str): The name of the App Connector Group.
            latitude (int): The latitude representing the App Connector's physical location.
            location (str): The name of the location that the App Connector Group represents.
            longitude (int): The longitude representing the App Connector's physical location.
            **kwargs: Optional keyword args.

        Keyword Args:
            **connector_ids (list):
                The unique ids for the App Connectors that will be added to this App Connector Group.
            **city_country (str):
                The City and Country for where the App Connectors are located. Format is:

                ``<City>, <Country Code>`` e.g. ``Sydney, AU``
            **country_code (str):
                The ISO<std> Country Code that represents the country where the App Connectors are located.
            **description (str):
                Additional information about the App Connector Group.
            **dns_query_type (str):
                The type of DNS queries that are enabled for this App Connector Group. Accepted values are:
                ``IPV4_IPV6``, ``IPV4`` and ``IPV6``
            **enabled (bool):
                Is the App Connector Group enabled? Defaults to ``True``.
            **override_version_profile (bool):
                Override the local App Connector version according to ``version_profile``. Defaults to ``False``.
            **server_group_ids (list):
                The unique ids of the Server Groups that are associated with this App Connector Group
            **lss_app_connector_group (bool):
            **upgrade_day (str):
                The day of the week that upgrades will be pushed to the App Connector.
            **upgrade_time_in_secs (str):
                The time of the day that upgrades will be pushed to the App Connector.
            **version_profile (str):
                The version profile to use. This will automatically set ``override_version_profile`` to True.
                Accepted values are:
                ``default``, ``previous_default`` and ``new_release``

        Returns:
            :obj:`Box`: The resource record of the newly created App Connector Group.

        Examples:
            Add a new ZPA App Connector Group with parameters.

            >>> group = zpa.connectors.add_connector_group(name="New App Connector Group",
            ...    location="Sydney",
            ...    latitude="33.8688",
            ...    longitude="151.2093",
            ...    version_profile="default")

        """
        payload = {
            "name": name,
            "latitude": latitude,
            "location": location,
            "longitude": longitude,
        }

        # Perform formatting on simplified params
        add_id_groups(self.reformat_params, kwargs, payload)
        pick_version_profile(kwargs, payload)

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post("appConnectorGroup", json=payload)

    def update_connector_group(self, group_id: str, **kwargs) -> Box:
        """
        Updates an existing ZPA App Connector Group.

        Args:
            group_id (str): The unique id for the App Connector Group in ZPA.
            **kwargs: Optional keyword args.

        Keyword Args:
            **connector_ids (list):
                The unique ids for the App Connectors that will be added to this App Connector Group.
            **city_country (str):
                The City and Country for where the App Connectors are located. Format is:

                ``<City>, <Country Code>`` e.g. ``Sydney, AU``
            **country_code (str):
                The ISO<std> Country Code that represents the country where the App Connectors are located.
            **description (str):
                Additional information about the App Connector Group.
            **dns_query_type (str):
                The type of DNS queries that are enabled for this App Connector Group. Accepted values are:
                ``IPV4_IPV6``, ``IPV4`` and ``IPV6``
            **enabled (bool):
                Is the App Connector Group enabled? Defaults to ``True``.
            **name (str): The name of the App Connector Group.
            **latitude (int): The latitude representing the App Connector's physical location.
            **location (str): The name of the location that the App Connector Group represents.
            **longitude (int): The longitude representing the App Connector's physical location.
            **override_version_profile (bool):
                Override the local App Connector version according to ``version_profile``. Defaults to ``False``.
            **server_group_ids (list):
                The unique ids of the Server Groups that are associated with this App Connector Group
            **lss_app_connector_group (bool):
            **upgrade_day (str):
                The day of the week that upgrades will be pushed to the App Connector.
            **upgrade_time_in_secs (str):
                The time of the day that upgrades will be pushed to the App Connector.
            **version_profile (str):
                The version profile to use. This will automatically set ``override_version_profile`` to True.
                Accepted values are:

                ``default``, ``previous_default`` and ``new_release``

        Returns:
            :obj:`Box`: The updated ZPA App Connector Group resource record.

        Examples:
            Update the name of an App Connector Group in ZPA, change the version profile to new releases and disable
            the group.

            >>> group = zpa.connectors.update_connector_group('99999',
            ...    name="Updated App Connector Group",
            ...    version_profile="new_release",
            ...    enabled=False)

        """

        # Set payload to equal existing record
        payload = {snake_to_camel(k): v for k, v in self.get_connector_group(group_id).items()}

        # Perform formatting on simplified params
        add_id_groups(self.reformat_params, kwargs, payload)
        pick_version_profile(kwargs, payload)

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"appConnectorGroup/{group_id}", json=payload).status_code

        if resp == 204:
            return self.get_connector_group(group_id)

    def delete_connector_group(self, group_id: str) -> int:
        """
        Deletes the specified App Connector Group from ZPA.

        Args:
            group_id (str): The unique identifier for the App Connector Group.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            >>> zpa.connectors.delete_connector_group('1876541121')

        """
        return self._delete(f"appConnectorGroup/{group_id}").status_code



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, snake_to_camel


class LocationsAPI(APIEndpoint):
    def list_locations(self, **kwargs) -> BoxList:
        """
        Returns a list of locations.

        Keyword Args:
            **auth_required (bool, optional):
                Filter based on whether the Enforce Authentication setting is enabled or disabled for a location.
            **bw_enforced (bool, optional):
                Filter based on whether Bandwith Control is being enforced for a location.
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **page_size (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to partially match against a location's name and port attributes.
            **xff_enabled (bool, optional):
                Filter based on whether the Enforce XFF Forwarding setting is enabled or disabled for a location.

        Returns:
            :obj:`BoxList`: List of configured locations.

        Examples:
            List locations using default settings:

            >>> for location in zia.locations.list_locations():
            ...    print(location)

            List locations, limiting to a maximum of 10 items:

            >>> for location in zia.locations.list_locations(max_items=10):
            ...    print(location)

            List locations, returning 200 items per page for a maximum of 2 pages:

            >>> for location in zia.locations.list_locations(page_size=200, max_pages=2):
            ...    print(location)

        """
        return BoxList(Iterator(self._api, "locations", **kwargs))

    def add_location(self, name: str, **kwargs) -> Box:
        """
        Adds a new location.

        Args:
            name (str):
                Location name.

        Keyword Args:
            parent_id (int, optional):
                Parent Location ID. If this ID does not exist or is 0, it is implied that it is a parent location.
            up_bandwidth (int, optional):
                Upload bandwidth in kbps. The value 0 implies no Bandwidth Control enforcement. Default: 0.
            dn_bandwidth (int, optional):
                Download bandwidth in kbps. The value 0 implies no Bandwidth Control enforcement. Default: 0.
            country (str, optional):
                Country.
            tz (str, optional):
                Timezone of the location. If not specified, it defaults to GMT.
            ip_addresses (list[str], optional):
                For locations: IP addresses of the egress points that are provisioned in the Zscaler Cloud.
                Each entry is a single IP address (e.g., 238.10.33.9).

                For sub-locations: Egress, internal, or GRE tunnel IP addresses. Each entry is either a single
                IP address, CIDR (e.g., 10.10.33.0/24), or range (e.g., 10.10.33.1-10.10.33.10)).
            ports (list[int], optional):
                IP ports that are associated with the location.
            vpn_credentials (list, optional):
                VPN User Credentials that are associated with the location.
            auth_required (bool, optional):
                Enforce Authentication. Required when ports are enabled, IP Surrogate is enabled, or Kerberos
                Authentication is enabled. Default: False.
            ssl_scan_enabled (bool, optional):
                Enable SSL Inspection. Set to true in order to apply your SSL Inspection policy to HTTPS traffic in
                the location and inspect HTTPS transactions for data leakage, malicious content, and viruses.
                Default: False.
            zapp_ssl_scan_enabled (bool, optional):
                Enable Zscaler App SSL Setting. When set to true, the Zscaler App SSL Scan Setting takes effect,
                irrespective of the SSL policy that is configured for the location. Default: False.
            xff_forward_enabled (bool, optional):
                Enable XFF Forwarding for a location. When set to true, traffic is passed to Zscaler Cloud via the
                X-Forwarded-For (XFF) header. Default: False.
            other_sub_location (bool, optional):
                If set to true, indicates that this is a default sub-location created by the Zscaler service to
                accommodate IPv4 addresses that are not part of any user-defined sub-locations. Default: False.
            other6_sub_location (bool, optional):
                If set to true, indicates that this is a default sub-location created by the Zscaler service to
                accommodate IPv6 addresses that are not part of any user-defined sub-locations. Default: False.
            surrogate_ip (bool, optional):
                Enable Surrogate IP. When set to true, users are mapped to internal device IP addresses. Default: False.
            idle_time_in_minutes (int, optional):
                Idle Time to Disassociation. The user mapping idle time (in minutes) is required if Surrogate IP is
                enabled.
            display_time_unit (str, optional):
                Display Time Unit. The time unit to display for IP Surrogate idle time to disassociation.
            surrogate_ip_enforced_for_known_browsers (bool, optional):
                Enforce Surrogate IP for Known Browsers. When set to true, IP Surrogate is enforced for all known
                browsers. Default: False.
            surrogate_refresh_time_in_minutes (int, optional):
                Refresh Time for re-validation of Surrogacy. The surrogate refresh time (in minutes) to re-validate
                the IP surrogates.
            surrogate_refresh_time_unit (str, optional):
                Display Refresh Time Unit. The time unit to display for refresh time for re-validation of surrogacy.
            ofw_enabled (bool, optional):
                Enable Firewall. When set to true, Firewall is enabled for the location. Default: False.
            ips_control (bool, optional):
                Enable IPS Control. When set to true, IPS Control is enabled for the location if Firewall is enabled.
                Default: False.
            aup_enabled (bool, optional):
                Enable AUP. When set to true, AUP is enabled for the location. Default: False.
            caution_enabled (bool, optional):
                Enable Caution. When set to true, a caution notification is enabled for the location. Default: False.
            aup_block_internet_until_accepted (bool, optional):
                For First Time AUP Behavior, Block Internet Access. When set, all internet access (including non-HTTP
                traffic) is disabled until the user accepts the AUP. Default: False.
            aup_force_ssl_inspection (bool, optional):
                For First Time AUP Behavior, Force SSL Inspection. When set, Zscaler forces SSL Inspection in order
                to enforce AUP for HTTPS traffic. Default: False.
            ipv6_enabled (bool, optional):
                If set to true, IPv6 is enabled for the location and IPv6 traffic from the location can be forwarded
                to the Zscaler service to enforce security policies.
            ipv6_dns64_prefix (str, optional):
                Name-ID pair of the NAT64 prefix configured as the DNS64 prefix for the location.
            aup_timeout_in_days (int, optional):
                Custom AUP Frequency. Refresh time (in days) to re-validate the AUP.
            managed_by (str, optional):
                SD-WAN Partner that manages the location. If a partner does not manage the location, this is set to
                Self.
            profile (str, optional):
                Profile tag that specifies the location traffic type. If not specified, this tag defaults to
                "Unassigned".
            description (str, optional):
                Additional notes or information regarding the location or sub-location. The description cannot
                exceed 1024 characters.

        Returns:
            :obj:`Box`: The newly created location resource record

        Examples:
            Add a new location with an IP address.

            >>> zia.locations.add_location(name='new_location',
            ...    ip_addresses=['203.0.113.10'])

            Add a location with VPN credentials.

            >>> zia.locations.add_location(name='new_location',
            ...    vpn_credentials=[{'id': '99999', 'type': 'UFQDN'}])

        """
        payload = {
            "name": name,
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post("locations", json=payload)

    def get_location(self, location_id: str = None, location_name: str = None) -> Box:
        """
        Returns information for the specified location based on the location id or location name.

        Args:
            location_id (str, optional):
                The unique identifier for the location.
            location_name (str, optional):
                The unique name for the location.

        Returns:
            :obj:`Box`: The requested location resource record.

        Examples:
            >>> location = zia.locations.get_location('97456691')

            >>> location = zia.locations.get_location_name(name='stockholm_office')
        """
        if location_id and location_name:
            raise ValueError("TOO MANY ARGUMENTS: Expected either location_id or location_name. Both were provided.")
        elif location_name:
            location = (record for record in self.list_locations(search=location_name) if record.name == location_name)
            return next(location, None)

        return self._get(f"locations/{location_id}")

    def list_sub_locations(self, location_id: str, **kwargs) -> BoxList:
        """
        Returns sub-location information for the specified location ID.

        Args:
            location_id (str):
                The unique identifier for the parent location.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            **auth_required (bool, optional):
                Filter based on whether the Enforce Authentication setting is enabled or disabled for a location.
            **bw_enforced (bool, optional):
                Filter based on whether Bandwith Control is being enforced for a location.
            **enable_firewall (bool, optional):
                Filter based on whether Enable Firewall setting is enabled or disabled for a sub-location.
            **enforce_aup (bool, optional):
                Filter based on whether Enforce AUP setting is enabled or disabled for a sub-location.
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **page_size (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to partially match against a location's name and port attributes.
            **xff_enabled (bool, optional):
                Filter based on whether the Enforce XFF Forwarding setting is enabled or disabled for a location.

        Returns:
            :obj:`BoxList`: A list of sub-locations configured for the parent location.

        Examples:
            >>> for sub_location in zia.locations.list_sub_locations('97456691'):
            ...    pprint(sub_location)

        """
        return BoxList(Iterator(self._api, f"locations/{location_id}/sublocations", max_pages=1, **kwargs))

    def list_locations_lite(self, **kwargs) -> BoxList:
        """
        Returns only the name and ID of all configured locations.

        Keyword Args:
            **include_parent_locations (bool, optional):
                Only locations with sub-locations will be included in the response if `True`.
            **include_sub_locations (bool, optional):
                Sub-locations will be included in the response if `True`.
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **page_size (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.
            **search (str, optional):
                The search string used to partially match against a location's name and port attributes.

        Returns:
            :obj:`BoxList`: A list of configured locations.

        Examples:
            List locations with default settings:

            >>> for location in zia.locations.list_locations_lite():
            ...    print(location)

            List locations, limiting to a maximum of 10 items:

            >>> for location in zia.locations.list_locations_lite(max_items=10):
            ...    print(location)

            List locations, returning 200 items per page for a maximum of 2 pages:

            >>> for location in zia.locations.list_locations_lite(page_size=200, max_pages=2):
            ...    print(location)

        """
        return BoxList(Iterator(self._api, "locations/lite", **kwargs))

    def update_location(self, location_id: str, **kwargs) -> Box:
        """
        Update the specified location.

        Note: Changes are not additive and will replace existing values.

        Args:
            location_id (str):
                The unique identifier for the location you are updating.
            **kwargs:
                Optional keyword arguments.

        Keyword Args:
            name (str, optional):
                Location name.
            parent_id (int, optional):
                Parent Location ID. If this ID does not exist or is 0, it is implied that it is a parent location.
            up_bandwidth (int, optional):
                Upload bandwidth in kbps. The value 0 implies no Bandwidth Control enforcement.
            dn_bandwidth (int, optional):
                Download bandwidth in kbps. The value 0 implies no Bandwidth Control enforcement.
            country (str, optional):
                Country.
            tz (str, optional):
                Timezone of the location. If not specified, it defaults to GMT.
            ip_addresses (list[str], optional):
                For locations: IP addresses of the egress points that are provisioned in the Zscaler Cloud.
                Each entry is a single IP address (e.g., 238.10.33.9).

                For sub-locations: Egress, internal, or GRE tunnel IP addresses. Each entry is either a single
                IP address, CIDR (e.g., 10.10.33.0/24), or range (e.g., 10.10.33.1-10.10.33.10)).
            ports (list[int], optional):
                IP ports that are associated with the location.
            vpn_credentials (list, optional):
                VPN User Credentials that are associated with the location.
            auth_required (bool, optional):
                Enforce Authentication. Required when ports are enabled, IP Surrogate is enabled, or Kerberos
                 Authentication is enabled.
            ssl_scan_enabled (bool, optional):
                Enable SSL Inspection. Set to true in order to apply your SSL Inspection policy to HTTPS traffic in the
                location and inspect HTTPS transactions for data leakage, malicious content, and viruses.
            zapp_ssl_scan_enabled (bool, optional):
                Enable Zscaler App SSL Setting. When set to true, the Zscaler App SSL Scan Setting takes effect,
                irrespective of the SSL policy that is configured for the location.
            xff_forward_enabled (bool, optional):
                Enable XFF Forwarding for a location. When set to true, traffic is passed to Zscaler Cloud via the
                X-Forwarded-For (XFF) header.
            other_sub_location (bool, optional):
                If set to true, indicates that this is a default sub-location created by the Zscaler service to
                accommodate IPv4 addresses that are not part of any user-defined sub-locations.
            other6_sub_location (bool, optional):
                If set to true, indicates that this is a default sub-location created by the Zscaler service to
                accommodate IPv6 addresses that are not part of any user-defined sub-locations.
            surrogate_ip (bool, optional):
                Enable Surrogate IP. When set to true, users are mapped to internal device IP addresses.
            idle_time_in_minutes (int, optional):
                Idle Time to Disassociation. The user mapping idle time (in minutes) is required if a Surrogate IP is
                enabled.
            display_time_unit (str, optional):
                Display Time Unit. The time unit to display for IP Surrogate idle time to disassociation.
            surrogate_ip_enforced_for_known_browsers (bool, optional):
                Enforce Surrogate IP for Known Browsers. When set to true, IP Surrogate is enforced for all known
                browsers.
            surrogate_refresh_time_in_minutes (int, optional):
                Refresh Time for re-validation of Surrogacy. The surrogate refresh time (in minutes) to re-validate
                the IP surrogates.
            surrogate_refresh_time_unit (str, optional):
                Display Refresh Time Unit. The time unit to display for refresh time for re-validation of surrogacy.
            ofw_enabled (bool, optional):
                Enable Firewall. When set to true, Firewall is enabled for the location.
            ips_control (bool, optional):
                Enable IPS Control. When set to true, IPS Control is enabled for the location if Firewall is enabled.
            aup_enabled (bool, optional):
                Enable AUP. When set to true, AUP is enabled for the location.
            caution_enabled (bool, optional):
                Enable Caution. When set to true, a caution notification is enabled for the location.
            aup_block_internet_until_accepted (bool, optional):
                For First Time AUP Behavior, Block Internet Access. When set, all internet access (including non-HTTP
                traffic) is disabled until the user accepts the AUP.
            aup_force_ssl_inspection (bool, optional):
                For First Time AUP Behavior, Force SSL Inspection. When set, Zscaler forces SSL Inspection in order to
                enforce AUP for HTTPS traffic.
            ipv6_enabled (bool, optional):
                If set to true, IPv6 is enabled for the location and IPv6 traffic from the location can be forwarded
                to the Zscaler service to enforce security policies.
            ipv6_dns64_prefix (str, optional):
                Name-ID pair of the NAT64 prefix configured as the DNS64 prefix for the location.
            aup_timeout_in_days (int, optional):
                Custom AUP Frequency. Refresh time (in days) to re-validate the AUP.
            managed_by (str, optional):
                SD-WAN Partner that manages the location. If a partner does not manage the location, this is set to
                Self.
            profile (str, optional):
                Profile tag that specifies the location traffic type. If not specified, this tag defaults to
                "Unassigned".
            description (str, optional):
                Additional notes or information regarding the location or sub-location. The description cannot exceed
                1024 characters.

        Returns:
            :obj:`Box`: The updated resource record.

        Examples:
            Update the name of a location:

            >>> zia.locations.update_location('99999',
            ...    name='updated_location_name')

            Update the IP address of a location:

            >>> zia.locations.update_location('99999',
            ...    ip_addresses=['203.0.113.20'])

            Update the VPN credentials of a location:

            >>> zia.locations.update_location('99999',
            ...    vpn_credentials=[{'id': '88888', 'type': 'UFQDN'}])

        """
        # Set payload to value of existing record
        payload = {snake_to_camel(k): v for k, v in self.get_location(location_id).items()}

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        # Fixes edge case where the sublocation object is missing displayTimeUnit, which will result in a 500 error.
        if not payload.get("displayTimeUnit"):
            payload["displayTimeUnit"] = "MINUTE"

        return self._put(f"locations/{location_id}", json=payload)

    def delete_location(self, location_id: str) -> int:
        """
        Deletes the location or sub-location for the specified ID

        Args:
            location_id (str):
                The unique identifier for the location or sub-location.

        Returns:
            :obj:`int`: Response code for the operation.

        Examples:
            >>> zia.locations.delete_location('97456691')

        """
        return self._delete(f"locations/{location_id}", box=False).status_code

    def get_geo_by_coordinates(self, latitude: int, longitude: int) -> Box:
        """
        Retrieves the geographical data of the region or city that is located in the specified latitude and longitude
        coordinates. The geographical data includes the city name, state, country, geographical ID of the city and
        state, etc.

        Args:
            latitude (int): The latitude of the location.
            longitude (int): The longitude of the location.

        Returns:
            :obj:`Box`: The geographical data of the region or city that is located in the specified coordinates.

        Examples:
            Get the geographical data of the region or city that is located in the specified coordinates::

                print(zia.locations.get_geo_by_coordinates(37.3860517, -122.0838511))

        """
        payload = {"latitude": latitude, "longitude": longitude}
        return self._get("region/byGeoCoordinates", params=payload)

    def get_geo_by_ip(self, ip: str) -> Box:
        """
        Retrieves the geographical data of the region or city that is located in the specified IP address. The
        geographical data includes the city name, state, country, geographical ID of the city and state, etc.

        Args:
            ip (str): The IP address of the location.

        Returns:
            :obj:`Box`: The geographical data of the region or city that is located in the specified IP address.

        Examples:
            Get the geographical data of the region or city that is located in the specified IP address::

                print(zia.locations.get_geo_by_ip("8.8.8.8")
        """
        return self._get(f"region/byIPAddress/{ip}")

    def list_cities_by_name(self, **kwargs) -> BoxList:
        """
        Retrieves the list of cities (along with their geographical data) that match the prefix search. The geographical
         data includes the latitude and longitude coordinates of the city, geographical ID of the city and state,
         country, postal code, etc.

        Args:
            **kwargs: Optional keyword arguments.

        Keyword Args:
            prefix (str): The prefix string to search for cities.
            page (int): The page number of the results.
            page_size (int): The number of results per page.

        Returns:
            :obj:`BoxList`: The list of cities (along with their geographical data) that match the prefix search.

        Examples:
            Get the list of cities (along with their geographical data) that match the prefix search::

                for city in zia.locations.list_cities_by_name(prefix="San Jose"):
                    print(city)

        Notes:
            Very broad or generic search terms may return a large number of results which can take a long time to be
            returned. Ensure you narrow your search result as much as possible to avoid this.

        """
        return BoxList(Iterator(self._api, "region/search", **kwargs))



from box import Box, BoxList
from restfly import APIEndpoint

from pyzscaler.utils import convert_keys


class ZCONAdminAPI(APIEndpoint):
    def list_roles(self, **kwargs) -> BoxList:
        """
        List all existing admin roles.

        Keyword Args:
            include_auditor_role (bool): Include / exclude auditor roles in the response.
            include_partner_role (bool): Include / exclude partner roles in the response.
            include_api_roles (bool): Include / exclude API roles in the response.
            id (list): The ID of the roles to include.

        Returns:
            :obj:`BoxList`: The list of roles.

        Examples:
            Print all roles::

                for role in zcon.admin.list_roles():
                    print(role)

            Print all roles with additional parameters::

                for role in zcon.admin.list_roles(
                    include_auditor_role=True,
                    include_partner_role=True,
                    include_api_roles=True,
                ):
                    print(role)

        """
        return self._get("adminRoles")

    def get_role(self, role_id: str) -> Box:
        """
        Get details for a specific admin role.

        Args:
            role_id (str): The ID of the role to retrieve.

        Returns:
            :obj:`Box`: The role details.

        Examples:
            Print the details of a role::

                print(zcon.admin.get_role("123456789")

        """
        return self._get(f"adminRoles/{role_id}")

    def add_role(
        self,
        name: str,
        policy_access: str = "NONE",
        report_access: str = "NONE",
        username_access: str = "NONE",
        dashboard_access: str = "NONE",
        **kwargs,
    ):
        """
        Create a new admin role.

        Args:
            name (str): The name of the role.
            policy_access (str): The policy access level.
            report_access (str): The report access level.
            username_access (str): The username access level.
            dashboard_access (str): The dashboard access level.

        Keyword Args:
            feature_permissions_tuples (:obj:`List[Tuple[str, str]]`):
                A list of tuple pairs specifying the feature permissions. Each tuple contains the feature name
                (case-insensitive) and its access level.

                Accepted feature names (case-insensitive) are:

                - ``APIKEY_MANAGEMENT``
                - ``EDGE_CONNECTOR_CLOUD_PROVISIONING``
                - ``EDGE_CONNECTOR_LOCATION_MANAGEMENT``
                - ``EDGE_CONNECTOR_DASHBOARD``
                - ``EDGE_CONNECTOR_FORWARDING``
                - ``EDGE_CONNECTOR_TEMPLATE``
                - ``REMOTE_ASSISTANCE_MANAGEMENT``
                - ``EDGE_CONNECTOR_ADMIN_MANAGEMENT``
                - ``EDGE_CONNECTOR_NSS_CONFIGURATION``
            alerting_access (str): The alerting access level.
            analysis_access (str): The analysis access level.
            admin_acct_access (str): The admin account access level.
            device_info_access (str): The device info access level.

        Note:
            For access levels, the accepted values are:

            - ``NONE``
            - ``READ_ONLY``
            - ``READ_WRITE``


        Returns:
            :obj:`dict`: The newly created role.

        Examples:
            Minimum required arguments::

                zcon.admin.add_role(name="NewRole")

            Including keyword arguments::

                zcon.admin.add_role(
                    name="AdvancedRole",
                    policy_access="READ_ONLY",
                    feature_permissions_tuples=[
                        ("apikey_management", "read_only"),
                        ("EDGE_CONNECTOR_CLOUD_PROVISIONING", "NONE")
                    ],
                    alerting_access="READ_WRITE"
                )

        """
        payload = {
            "name": name,
            "role_type": "EDGE_CONNECTOR_ADMIN",
            "policy_access": policy_access,
            "report_access": report_access,
            "username_access": username_access,
            "dashboard_access": dashboard_access,
        }

        if feature_permissions_tuples := kwargs.pop("feature_permissions_tuples", None):
            payload["feature_permissions"] = {k.upper(): v for k, v in feature_permissions_tuples}

        # Add optional parameters to payload
        payload.update({k: v for k, v in kwargs.items() if v is not None})

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._post("adminRoles", json=payload)

    def update_role(self, role_id: str, **kwargs) -> Box:
        """
        Update an existing admin role.

        Args:
            role_id (str): The ID of the role to update.

        Keyword Args:
            name (str): The name of the role.
            policy_access (str): The policy access level.
            report_access (str): The report access level.
            username_access (str): The username access level.
            dashboard_access (str): The dashboard access level.
            feature_permissions (:obj:`List[Tuple[str, str]]`):
                A list of tuple pairs specifying the feature permissions. Each tuple contains the feature name
                (case-insensitive) and its access level.

                Accepted feature names (case-insensitive) are:

                - ``APIKEY_MANAGEMENT``
                - ``EDGE_CONNECTOR_CLOUD_PROVISIONING``
                - ``EDGE_CONNECTOR_LOCATION_MANAGEMENT``
                - ``EDGE_CONNECTOR_DASHBOARD``
                - ``EDGE_CONNECTOR_FORWARDING``
                - ``EDGE_CONNECTOR_TEMPLATE``
                - ``REMOTE_ASSISTANCE_MANAGEMENT``
                - ``EDGE_CONNECTOR_ADMIN_MANAGEMENT``
                - ``EDGE_CONNECTOR_NSS_CONFIGURATION``
            alerting_access (str): The alerting access level.
            analysis_access (str): The analysis access level.
            admin_acct_access (str): The admin account access level.
            device_info_access (str): The device info access level.

        Note:
            For access levels, the accepted values are:

            - ``NONE``
            - ``READ_ONLY``
            - ``READ_WRITE``

        Returns:
            :obj:`Box`: The updated role.

        Examples:
            Update a role::

                zcon.admin.update_role(
                    role_id="123456789",
                    policy_access="READ_ONLY",
                    feature_permissions=[
                        ("apikey_management", "read_only"),
                        ("EDGE_CONNECTOR_CLOUD_PROVISIONING", "NONE")
                    ],
                    alerting_access="READ_WRITE"
                )

        """
        payload = self.get_role(role_id)

        # Pop the feature permissions out first so that we retain their format
        feature_permissions = kwargs.pop("feature_permissions", None)

        # Add optional parameters to payload
        payload.update({k: v for k, v in kwargs.items() if v is not None})

        # Convert snake to camelcase
        payload = convert_keys(payload)

        # Now update the feature permissions
        if feature_permissions:
            payload["featurePermissions"] = {k.upper(): v for k, v in feature_permissions}

        return self._put(f"adminRoles/{role_id}", json=payload)

    def delete_role(self, role_id: str) -> int:
        """
        Delete the specified admin role.

        Args:
            role_id (str): The ID of the role to delete.

        Returns:
            :obj:`int`: The status code of the operation.

        Examples:
            Delete a role::

                zcon.admin.delete_role("123456789")

        """
        return self._delete(f"adminRoles/{role_id}").status_code

    def change_password(self, username: str, old_password: str, new_password: str) -> int:
        """
        Change the password for the specified admin user.

        Args:
            username (str): The username of the admin user.
            old_password (str): The current password of the admin user.
            new_password (str): The new password for the admin user.

        Returns:
            :obj:`int`: The status code of the operation.

        Examples:
            Change a password::

                zcon.admin.change_password("jdoe", "oldpassword123", "newpassword123")

        """
        payload = {
            "userName": username,
            "oldPassword": old_password,
            "newPassword": new_password,
        }
        return self._post("passwordChange", json=payload).status_code

    def list_admins(self, **kwargs) -> BoxList:
        """
        List all existing admin users.

        Keyword Args:
            include_auditor_users (bool): Include / exclude auditor users in the response.
            include_admin_users (bool): Include / exclude admin users in the response.
            include_api_roles (bool): Include / exclude API roles in the response.
            search (str): The search string to filter by.
            page (int): The page offset to return.
            page_size (int): The number of records to return per page.
            version (int): Specifies the admins from a backup version


        Returns:
            :obj:`BoxList`: The list of admin users.

        Examples:
            List all admins::

                for admin in zcon.admin.list_admins():
                    print(admin)

            List all admins with advanced features::

                for admin in zcon.admin.list_admins(
                    include_auditor_users=True,
                    include_admin_users=True,
                    include_api_roles=True,
                ):
                    print(admin)

        """
        payload = {
            "partnerType": "EDGE_CONNECTOR_ADMIN",
        }

        # Update the payload with keyword arguments
        payload.update({k: v for k, v in kwargs.items() if v is not None})

        # Convert snake to camelcase if needed
        payload = convert_keys(payload)

        return self._get("adminUsers", params=payload)

    def get_admin(self, admin_id: str) -> Box:
        """
        Get details for a specific admin user.

        Args:
            admin_id (str): The ID of the admin user to retrieve.

        Returns:
            :obj:`Box`: The admin user details.

        Examples:
            Print the details of an admin user::

                print(zcon.admin.get_admin("123456789")

        """
        return self._get(f"adminUsers/{admin_id}")

    def add_admin(self, user_name: str, login_name: str, role: str, email: str, password: str, **kwargs) -> Box:
        """
        Create a new admin user.

        Args:
            user_name (str): The name of the admin user.
            login_name (str): The login name of the admin user.
            role (str): The role of the admin user.
            email (str): The email address of the admin user.
            password (str): The password for the admin user.

        Keyword Args:
            disabled (bool): Indicates whether the admin is disabled.
            new_location_create_allowed (bool): Indicates whether the admin can create new locations.
            admin_scope_type (str): The admin scope type.
            admin_scope_group_member_entity_ids (list): Applicable if the admin scope type is `LOCATION_GROUP`.
            is_default_admin (bool): Indicates whether the admin is the default admin.
            is_deprecated_default_admin (bool): Indicates whether this admin is deletable.
            is_auditor (bool): Indicates whether the admin is an auditor.
            is_security_report_comm_enabled (bool): Indicates whether the admin can receive security reports.
            is_service_update_comm_enabled (bool): Indicates whether the admin can receive service updates.
            is_password_login_allowed (bool): Indicates whether the admin can log in with a password.
            is_product_update_comm_enabled (bool): Indicates whether the admin can receive product updates.
            is_exec_mobile_app_enabled (bool): Indicates whether Executive Insights App access is enabled for the admin.
            send_mobile_app_invite (bool):
                Indicates whether to send an invitation email to download Executive Insights App to admin.
            send_zdx_onboard_invite (bool): Indicates whether to send an invitation email for ZDX onboarding to admin.
            comments (str): Comments for the admin user.
            name (str):
                Admin user's "friendly" name, e.g., "FirstName LastName" (this field typically matches userName.)

        Returns:
            Box: A Box object representing the newly created admin user.

        Examples:
            Create a new admin user with only the required parameters::

                zcon.admin.add_admin(
                    name="Jane Smith",
                    login_name="jsmith",
                    role="admin",
                    email="jsmith@example.com",
                    password="password123",
                    )

            Create a new admin with some additional parameters::

                zcon.admin.add_admin(
                    name="Jane Smith",
                    login_name="jsmith",
                    role="admin",
                    email="jsmith@example.com",
                    password="password123",
                    is_default_admin=False,
                    disabled=False,
                    comments="New admin user"

        """

        payload = {
            "loginName": login_name,
            "userName": user_name,
            "email": email,
            "role": role,
            "password": password,
        }

        # Add optional parameters to payload
        payload.update({k: v for k, v in kwargs.items() if v is not None})

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._post("adminUsers", json=payload)

    def update_admin(self, admin_id: str, **kwargs) -> Box:
        """
        Update an existing admin user.

        Args:
            admin_id (str): The ID of the admin user to update.

        Keyword Args:
            role (str): The role of the admin user.
            email (str): The email address of the admin user.
            password (str): The password for the admin user.
            disabled (bool): Indicates whether the admin is disabled.
            new_location_create_allowed (bool): Indicates whether the admin can create new locations.
            admin_scope_type (str): The admin scope type.
            admin_scope_group_member_entity_ids (list): Applicable if the admin scope type is `LOCATION_GROUP`.
            is_default_admin (bool): Indicates whether the admin is the default admin.
            is_deprecated_default_admin (bool): Indicates whether this admin is deletable.
            is_auditor (bool): Indicates whether the admin is an auditor.
            is_security_report_comm_enabled (bool): Indicates whether the admin can receive security reports.
            is_service_update_comm_enabled (bool): Indicates whether the admin can receive service updates.
            is_password_login_allowed (bool): Indicates whether the admin can log in with a password.
            is_product_update_comm_enabled (bool): Indicates whether the admin can receive product updates.
            is_exec_mobile_app_enabled (bool): Indicates whether Executive Insights App access is enabled for the admin.
            send_mobile_app_invite (bool):
                Indicates whether to send an invitation email to download Executive Insights App to admin.
            send_zdx_onboard_invite (bool): Indicates whether to send an invitation email for ZDX onboarding to admin.
            comments (str): Comments for the admin user.
            name (str):
                Admin user's "friendly" name, e.g., "FirstName LastName" (this field typically matches userName.)

        Returns:
            Box: A Box object representing the updated admin user.

        Examples:
            Update an admin user::

                zcon.admin.update_admin(
                    admin_id="123456789",
                    admin_scope_type="LOCATION_GROUP",
                    comments="Updated admin user",
                )

        """

        payload = self.get_admin(admin_id)

        # Add optional parameters to payload
        payload.update({k: v for k, v in kwargs.items() if v is not None})

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._put(f"adminUsers/{admin_id}", json=payload)

    def delete_admin(self, admin_id: str) -> int:
        """
        Delete the specified admin user.

        Args:
            admin_id (str): The ID of the admin user to delete.

        Returns:
            :obj:`int`: The status code of the operation.

        Examples:
            Delete an admin user::

                zcon.admin.delete_admin("123456789")

        """
        return self._delete(f"adminUsers/{admin_id}").status_code

    def list_api_keys(self, **kwargs) -> BoxList:
        """
        List all existing API keys.

        Keyword Args:
            include_partner_keys (bool): Include / exclude partner keys in the response.

        Returns:
            :obj:`BoxList`: The list of API keys.

        Examples:
            List all API keys::

                for api_key in zcon.admin.list_api_keys():
                    print(api_key)
        """
        params = {}
        if "include_partner_keys" in kwargs:
            params["includePartnerKeys"] = kwargs["include_partner_keys"]

        return self._get("apiKeys", params=params)

    def regenerate_api_key(self, api_key_id: str) -> Box:
        """
        Regenerate the specified API key.

        Args:
            api_key_id (str): The ID of the API key to regenerate.

        Returns:
            :obj:`Box`: The regenerated API key.

        Examples:
            Regenerate an API key::

                print(zcon.admin.regenerate_api_key("123456789"))

        """
        return self._post(f"apiKeys/{api_key_id}/regenerate")



from box import Box
from restfly import APIEndpoint


class ZCONConfigAPI(APIEndpoint):
    def activate(self, force: bool = False) -> Box:
        """
        Activate the configuration.

        Args:
            force (bool): If set to True, forces the activation. Default is False.

        Returns:
            :obj:`Box`: The status code of the operation.

        Examples:
            Activate the configuration without forcing::

                zcon.config.activate()

            Forcefully activate the configuration::

                zcon.config.activate(force=True)

        """
        if force:
            return self._post("ecAdminActivateStatus/forcedActivate")
        else:
            return self._post("ecAdminActivateStatus/activate")

    def get_status(self):
        """
        Get the status of the configuration.

        Returns:
            :obj:`Box`: The status of the configuration.

        Examples:
            Get the status of the configuration::

                print(zcon.config.get_status())

        """
        return self._get("ecAdminActivateStatus")



from box import Box, BoxList
from restfly import APIEndpoint

from pyzscaler.utils import convert_keys


class ZCONConnectorsAPI(APIEndpoint):
    def list_groups(self, **kwargs) -> BoxList:
        """
        List all existing connector groups.

        Keyword Args:
            page (int): The page number to return.
            page_size (int): The number of items to return per page.

        Returns:
            :obj:`BoxList`: The list of cloud and branch connector groups.

        Examples:
            List all connector groups::

                for group in zcon.connectors.list_groups():
                    print(group)

            List first page of connector groups with 10 items per page::

                for group in zcon.connectors.list_groups(page=1, page_size=10):
                    print(group)

        """
        # There is an edge case in the camelcase to snake conversion that we're going to handle here. We'll revert
        # the default camel_killer_box and run it through our conversion function in utils that handles edge-cases.
        self._box_attrs = {"camel_killer_box": False}

        return convert_keys(self._get("ecgroup", params=kwargs), "to_snake")

    def get_group(self, group_id: str) -> Box:
        """
        Get details for a specific connector group.

        Args:
            group_id (str): The ID of the connector group.

        Returns:
            :obj:`Box`: The connector group details.

        Examples:
            Get details of a specific connector group::

                print(zcon.connectors.get_group("123456789"))
        """
        # There is an edge case in the camelcase to snake conversion that we're going to handle here. We'll revert
        # the default camel_killer_box and run it through our conversion function in utils that handles edge-cases.
        self._box_attrs = {"camel_killer_box": False}

        return convert_keys(self._get(f"ecgroup/{group_id}"), "to_snake")

    def get_vm(self, group_id: str, vm_id: str) -> Box:
        """
        Get details for a specific connector VM.

        Args:
            group_id (str): The ID of the connector group.
            vm_id (str): The ID of the connector VM.

        Returns:
            :obj:`Box`: The connector VM details.

        Examples:
            Get details of a specific connector VM::

                print(zcon.connectors.get_vm("123456789", "987654321"))
        """
        return self._get(f"ecgroup/{group_id}/vm/{vm_id}")

    def delete_vm(self, group_id: str, vm_id: str) -> int:
        """
        Delete the specified connector VM.

        Args:
            group_id (str): The ID of the connector group.
            vm_id (str): The ID of the connector VM.

        Returns:
            :obj:`Box`: The status of the operation.

        Examples:
            Delete a specific connector VM::

                zcon.connectors.delete_vm("123456789", "987654321")
        """
        return self._delete(f"ecgroup/{group_id}/vm/{vm_id}").status_code



from box import Box, BoxList
from restfly import APIEndpoint

from pyzscaler.utils import convert_keys


class ZCONLocationsAPI(APIEndpoint):
    def list_locations(self, **kwargs) -> BoxList:
        """
        List all existing locations.

        Keyword Args:
            group_id (str): The ID of the connector group.
            search (str): The search string to filter the results.
            state (str): The geographical state of the location.
            ssl_scan_enabled (bool): Include / exclude locations with SSL scanning enabled.
            xff_enabled (bool): Include / exclude locations with XFF enabled.
            auth_required (bool): Include / exclude locations with authentication required.
            bw_enforced (bool): Include / exclude locations with bandwidth enforcement enabled.
            partner_id (str): The ID of the partner. Not used for Cloud/Branch connector
            enforce_aup (bool): Include / exclude locations with AUP enforcement enabled.
            enable_firewall (bool): Include / exclude locations with firewall enabled.
            location_type (str): The type of location, accepted values are:

                - ``NONE``
                - ``CORPORATE``
                - ``SERVER``
                - ``GUESTWIFI``
                - ``IOT``
                - ``WORKLOAD``
            page (int): The page number to return.
            page_size (int): The number of items to return per page.

        Returns:
            :obj:`BoxList`: The list of connector locations.

        Examples:
            List all locations::

                for location in zcon.locations.list_locations():
                    print(location)

            List only IOT locations::

                for location in zcon.locations.list_locations(location_type="IOT"):
                    print(location)

        """
        return self._get("location")

    def get_location(self, location_id: str) -> Box:
        """
        Get details for a specific location.

        Args:
            location_id (str): The ID of the location to retrieve.

        Returns:
            :obj:`Box`: The location details.

        Examples:
            Get details of a specific location::

                print(zcon.locations.get_location("123456789"))

        """
        return self._get(f"adminRoles/{location_id}")

    def list_location_templates(self, **kwargs) -> BoxList:
        """
        List all existing location templates.

        Args:
            **kwargs: Optional keyword args to filter the results.

        Keyword Args:
            page (int): The page number to return.
            page_size (int): The number of items to return per page.

        Returns:
            :obj:`BoxList`: The list of location templates.

        Examples:
            List all location templates::

                for template in zcon.locations.list_location_templates():
                    print(template)

        """
        return self._get("locationTemplate", params=kwargs)

    def get_location_template(self, template_id: str) -> Box:
        """
        Get details for a specific location template.

        Args:
            template_id (str): The ID of the location template to retrieve.

        Returns:
            :obj:`Box`: The location template details.

        Examples:
            Get details of a specific location template::

                print(zcon.locations.get_location_template("123456789"))

        """
        return self._get(f"locationTemplate/{template_id}")

    def add_location_template(self, name: str, template: dict = None, **kwargs) -> Box:
        """
        Add a new location template.

        Args:
            name (str): The name of the location template.
            template (dict, optional): A dictionary containing the template settings. Possible keys include:

                - ``template_prefix`` (str): Prefix of Cloud & Branch Connector location template.
                - ``xff_forward_enabled`` (bool): Enable to use the X-Forwarded-For headers.
                - ``auth_required`` (bool): Enable if "Authentication Required" is needed.
                - ``caution_enabled`` (bool): Enable to display an end user notification for unauthenticated traffic.
                - ``aup_enabled`` (bool): Enable to display an Acceptable Use Policy (AUP) for unauthenticated traffic.
                - ``aup_timeout_in_days`` (int): Frequency in days for displaying the AUP, if enabled.
                - ``ofw_enabled`` (bool): Enable the service's firewall controls.
                - ``ips_control`` (bool): Enable IPS controls, if firewall is enabled.
                - ``enforce_bandwidth_control`` (bool): Enable to specify bandwidth limits.
                - ``up_bandwidth`` (int): Upload bandwidth in Mbps, if bandwidth control is enabled.
                - ``dn_bandwidth`` (int): Download bandwidth in Mbps, if bandwidth control is enabled.
                - ``display_time_unit`` (str): Time unit for IP Surrogate idle time to disassociation.
                - ``idle_time_in_minutes`` (int): User mapping idle time in minutes for IP Surrogate.
                - ``surrogate_ip_enforced_for_known_browsers`` (bool): Enforce IP Surrogate for all known browsers.
                - ``surrogate_refresh_time_unit`` (str): Time unit for refresh time for re-validation of surrogacy.
                - ``surrogate_refresh_time_in_minutes`` (int): Refresh time in minutes for re-validation of surrogacy.
                - ``surrogate_ip`` (bool): Enable the IP Surrogate feature.

        Keyword Args:
            description (str): The description of the location template.

        Returns:
            :obj:`Box`: The location template details.

        Examples:
            Add a new location template with minimal settings::

                print(zcon.locations.add_location_template(name="MyTemplate"))

            Add a new location template with additional settings::

                template_settings = {
                    "surrogate_ip": True,
                    "surrogate_ip_enforced_for_known_browsers": False,
                    "template_prefix": "office",
                    "aup_enabled": True,
                    "aup_timeout_in_days": 30,
                    "ofw_enabled": True,
                    "idle_time_in_minutes": 35,
                    "auth_required": True,
                    "display_time_unit": "MINUTE",
                }
                print(zcon.locations.add_location_template(name="MyTemplate", template=template_settings))
        """
        # Rename 'description' to 'desc' if it exists
        if "description" in kwargs:
            kwargs["desc"] = kwargs.pop("description")

        payload = {"name": name, "template": template if template is not None else {}}

        # Add optional parameters to payload
        payload.update({k: v for k, v in kwargs.items() if v is not None})

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._post("locationTemplate", json=payload)

    def update_location_template(self, template_id: str, **kwargs) -> Box:
        """
        Update an existing location template.

        Args:
            template_id (str): The ID of the location template to update.

        Keyword Args:
            name (str): The name of the location template.
            template (dict): A dictionary containing the template settings. Possible keys include:

                - ``template_prefix`` (str): Prefix of Cloud & Branch Connector location template.
                - ``xff_forward_enabled`` (bool): Enable to use the X-Forwarded-For headers.
                - ``auth_required`` (bool): Enable if "Authentication Required" is needed.
                - ``caution_enabled`` (bool): Enable to display an end user notification for unauthenticated traffic.
                - ``aup_enabled`` (bool): Enable to display an Acceptable Use Policy (AUP) for unauthenticated traffic.
                - ``aup_timeout_in_days`` (int): Frequency in days for displaying the AUP, if enabled.
                - ``ofw_enabled`` (bool): Enable the service's firewall controls.
                - ``ips_control`` (bool): Enable IPS controls, if firewall is enabled.
                - ``enforce_bandwidth_control`` (bool): Enable to specify bandwidth limits.
                - ``up_bandwidth`` (int): Upload bandwidth in Mbps, if bandwidth control is enabled.
                - ``dn_bandwidth`` (int): Download bandwidth in Mbps, if bandwidth control is enabled.
                - ``display_time_unit`` (str): Time unit for IP Surrogate idle time to disassociation.
                - ``idle_time_in_minutes`` (int): User mapping idle time in minutes for IP Surrogate.
                - ``surrogate_ip_enforced_for_known_browsers`` (bool): Enforce IP Surrogate for all known browsers.
                - ``surrogate_refresh_time_unit`` (str): Time unit for refresh time for re-validation of surrogacy.
                - ``surrogate_refresh_time_in_minutes`` (int): Refresh time in minutes for re-validation of surrogacy.
                - ``surrogate_ip`` (bool): Enable the IP Surrogate feature.
            description (str): A description for the location template.

        Returns:
            :obj:`Box`: The updated location template details.

        Note:
            - Any provided keys will update existing keys.
            - The template dictionary does not support partial updates. Any provided template will completely overwrite
                the existing template.

        Examples:
            Update the name of a location template::

                print(zcon.locations.update_location_template(template_id="123456789", name="MyTemplate"))

            Update the template details of a location template::

                template_settings = {
                    "surrogate_ip": True,
                    "surrogate_ip_enforced_for_known_browsers": False,
                    "template_prefix": "office",
                    "aup_enabled": True,
                    "aup_timeout_in_days": 30,
                    "ofw_enabled": True,
                    "idle_time_in_minutes": 4, # <-- changed to 4 hours
                    "auth_required": True,
                    "display_time_unit": "HOUR", # <-- changed from minutes to hours
                }
                print(zcon.locations.update_location_template(template_id="123456789", template=template_settings))
        """

        # Rename 'description' to 'desc' if it exists
        if "description" in kwargs:
            kwargs["desc"] = kwargs.pop("description")

        # Retrieve existing location template
        payload = self.get_location_template(template_id)

        # Merge all kwargs into payload
        payload.update(convert_keys(kwargs))

        return self._put(f"locationTemplate/{template_id}", json=payload)

    def delete_location_template(self, template_id: str):
        """
        Delete an existing location template.

        Args:
            template_id (str): The ID of the location template to delete.

        Returns:
            :obj:`int`: The status code of the operation.

        Examples:
            Delete a location template::

                print(zcon.locations.delete_location_template("123456789"))
        """
        return self._delete(f"locationTemplate/{template_id}").status_code



from restfly.endpoint import APIEndpoint

from pyzscaler.utils import zdx_params


class DevicesAPI(APIEndpoint):
    @zdx_params
    def list_devices(self, **kwargs):
        """
        Returns a list of all devices in ZDX.

        Keyword Args:
            since (int): The number of hours to look back for devices.
            location_id (str): The unique ID for the location.
            department_id (str): The unique ID for the department.
            geo_id (str): The unique ID for the geolocation.

        Returns:
            :obj:`BoxList`: The list of devices in ZDX.

        Examples:
            List all devices in ZDX for the past 2 hours:

            >>> for device in zdx.devices.list_devices():

            List all devices in ZDX for the past 24 hours:

            >>> for device in zdx.devices.list_devices(since=24):

        """
        return self._get("devices", params=kwargs)

    @zdx_params
    def get_device(self, device_id: str, **kwargs):
        """
        Returns a single device in ZDX.

        Args:
            device_id (str): The unique ID for the device.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The ZDX device resource record.

        Examples:
            Get information for the device with an ID of 123456789.
            >>> device = zdx.devices.get_device('123456789')

            Get information for the device with an ID of 123456789 for the last 24 hours.
            >>> device = zdx.devices.get_device('123456789', since=24)

        """
        return self._get(f"devices/{device_id}", params=kwargs)

    @zdx_params
    def get_device_apps(self, device_id: str, **kwargs):
        """
        Returns a list of all active applications for a device.

        Args:
            device_id (str): The unique ID for the device.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`BoxList`: The list of active applications for the device.

        Examples:
            Print a list of active applications for a device.

            >>> for app in zdx.devices.get_device_apps('123456789'):
            ...     print(app)

            Print a list of active applications for a device for the last 24 hours.

            >>> for app in zdx.devices.get_device_apps('123456789', since=24):
            ...     print(app)

        """
        return self._get(f"devices/{device_id}/apps", params=kwargs)

    def get_device_app(self, device_id: str, app_id: str):
        """
        Returns a single application for a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.

        Returns:
            :obj:`Box`: The application resource record.

        Examples:
            Print a single application for a device.

            >>> app = zdx.devices.get_device_app('123456789', '987654321')
            ... print(app)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}")

    def get_web_probes(self, device_id: str, app_id: str):
        """
        Returns a list of all active web probes for a specific application being used by a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.

        Returns:
            :obj:`BoxList`: The list of web probes for the application.

        Examples:
            Print a list of web probes for an application.

            >>> for probe in zdx.devices.get_device_app_webprobes('123456789', '987654321'):
            ...     print(probe)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}/web-probes")

    @zdx_params
    def get_web_probe(self, device_id: str, app_id: str, probe_id: str, **kwargs):
        """
        Returns a single web probe for a specific application being used by a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.
            probe_id (str): The unique ID for the web probe.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The web probe resource record.

        Examples:
            Print a single web probe for an application.

            >>> probe = zdx.devices.get_web_probe('123456789', '987654321', '123987456')
            ... print(probe)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}/web-probes/{probe_id}", params=kwargs)

    @zdx_params
    def list_cloudpath_probes(self, device_id: str, app_id: str, **kwargs):
        """
        Returns a list of all active cloudpath probes for a specific application being used by a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`BoxList`: The list of cloudpath probes for the application.

        Examples:
            Print a list of cloudpath probes for an application.

            >>> for probe in zdx.devices.list_cloudpath_probes('123456789', '987654321'):
            ...     print(probe)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}/cloudpath-probes", params=kwargs)

    @zdx_params
    def get_cloudpath_probe(self, device_id: str, app_id: str, probe_id: str, **kwargs):
        """
        Returns a single cloudpath probe for a specific application being used by a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.
            probe_id (str): The unique ID for the cloudpath probe.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The cloudpath probe resource record.

        Examples:
            Print a single cloudpath probe for an application.

            >>> probe = zdx.devices.get_cloudpath_probe('123456789', '987654321', '123987456')
            ... print(probe)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}/cloudpath-probes/{probe_id}", params=kwargs)

    @zdx_params
    def get_cloudpath(self, device_id: str, app_id: str, probe_id: str, **kwargs):
        """
        Returns a single cloudpath for a specific application being used by a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.
            probe_id (str): The unique ID for the cloudpath probe.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The cloudpath resource record.

        Examples:
            Print a single cloudpath for an application.

            >>> cloudpath = zdx.devices.get_cloudpath('123456789', '987654321', '123987456')
            ... print(cloudpath)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}/cloudpath-probes/{probe_id}/cloudpath", params=kwargs)

    @zdx_params
    def get_call_quality_metrics(self, device_id: str, app_id: str, **kwargs):
        """
        Returns a single call quality metrics for a specific application being used by a device.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The call quality metrics resource record.

        Examples:
            Print call quality metrics for an application.

            >>> metrics = zdx.devices.get_call_quality_metrics('123456789', '987654321')
            ... print(metrics)

        """
        return self._get(f"devices/{device_id}/apps/{app_id}/call-quality-metrics", params=kwargs)

    @zdx_params
    def get_health_metrics(self, device_id: str, **kwargs):
        """
        Returns health metrics trend for a specific device.

        Args:
            device_id (str): The unique ID for the device.

        Keyword Args:
            since (int): The number of hours to look back for devices.

        Returns:
            :obj:`Box`: The health metrics resource record.

        Examples:
            Print health metrics for an application.

            >>> metrics = zdx.devices.get_health_metrics('123456789')
            ... print(metrics)

        """
        return self._get(f"devices/{device_id}/health-metrics", params=kwargs)

    def get_events(self, device_id: str):
        """
        Returns a list of all events for a specific device.

        Args:
            device_id (str): The unique ID for the device.

        Returns:
            :obj:`BoxList`: The list of events for the device.

        Examples:
            Print a list of events for a device.

            >>> for event in zdx.devices.get_events('123456789'):
            ...     print(event)

        """
        return self._get(f"devices/{device_id}/events")

    def list_deeptraces(self, device_id: str):
        """
        Returns a list of all deep traces for a specific device.

        Args:
            device_id (str): The unique ID for the device.

        Returns:
            :obj:`BoxList`: The list of deep traces for the device.

        Examples:
            Print a list of deep traces for a device.

            >>> for trace in zdx.devices.list_deep_traces('123456789'):
            ...     print(trace)

        """
        return self._get(f"devices/{device_id}/deeptraces")

    def start_deeptrace(self, device_id: str, app_id: str, session_name: str, **kwargs):
        """
        Starts a deep trace for a specific device and application.

        Args:
            device_id (str): The unique ID for the device.
            app_id (str): The unique ID for the application.
            session_name (str): The name of the deeptrace session.

        Keyword Args:
            web_probe_id (str): The unique ID for the Web probe.
            cloudpath_probe_id (str): The unique ID for the Cloudpath probe.
            session_length_minutes (int): The duration of the deeptrace session in minutes. Defaults to 5.
            probe_device (bool): Whether to probe the device.

        Returns:
            :obj:`Box`: The deeptrace resource record.

        Examples:
            Start a deeptrace for a device.

            >>> trace = zdx.devices.start_deeptrace(device_id='123456789', app_id='1', session_name='My Deeptrace')
            ... print(trace)

        """
        payload = {
            "session_name": session_name,
            "app_id": app_id,
        }
        payload.update(kwargs)

        return self._post(f"devices/{device_id}/deeptraces", json=payload)

    def get_deeptrace(self, device_id: str, trace_id: str):
        """
        Returns information on a single deeptrace for a specific device.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace resource record.

        Examples:
            Print a single deeptrace for a device.

            >>> trace = zdx.devices.get_deeptrace('123456789', '987654321')
            ... print(trace)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}")

    def delete_deeptrace(self, device_id: str, trace_id: str):
        """
        Deletes a single deeptrace session and associated data for a specific device.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`str`: The trace ID that was deleted.

        Examples:
            Delete a single deeptrace for a device.

            >>> trace = zdx.devices.delete_deeptrace('123456789', '987654321')
            ... print(trace)

        """
        return self._delete(f"devices/{device_id}/deeptraces/{trace_id}")

    def get_deeptrace_webprobe_metrics(self, device_id: str, trace_id: str):
        """
        Returns web probe metrics for a specific deeptrace.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace web probe metrics.

        Examples:
            Print web probe metrics for a deeptrace.

            >>> metrics = zdx.devices.get_deeptrace_webprobe_metrics('123456789', '987654321')
            ... print(metrics)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}/webprobe-metrics")

    def get_deeptrace_cloudpath_metrics(self, device_id: str, trace_id: str):
        """
        Returns cloudpath metrics for a specific deeptrace.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace cloudpath metrics.

        Examples:
            Print cloudpath metrics for a deeptrace.

            >>> metrics = zdx.devices.get_deeptrace_cloudpath_metrics('123456789', '987654321')
            ... print(metrics)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}/cloudpath-metrics")

    def get_deeptrace_cloudpath(self, device_id: str, trace_id: str):
        """
        Returns cloudpath for a specific deeptrace.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace cloudpath.

        Examples:
            Print cloudpath for a deeptrace.

            >>> metrics = zdx.devices.get_deeptrace_cloudpath('123456789', '987654321')
            ... print(metrics)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}/cloudpath")

    def get_deeptrace_health_metrics(self, device_id: str, trace_id: str):
        """
        Returns health metrics for a specific deeptrace.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace health metrics.

        Examples:
            Print health metrics for a deeptrace.

            >>> metrics = zdx.devices.get_deeptrace_health_metrics('123456789', '987654321')
            ... print(metrics)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}/health-metrics")

    def get_deeptrace_events(self, device_id: str, trace_id: str):
        """
        Returns events for a specific deeptrace.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace events.

        Examples:
            Print events for a deeptrace.

            >>> events = zdx.devices.get_deeptrace_events('123456789', '987654321')
            ... print(events)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}/events")

    def get_deeptrace_top_processes(self, device_id: str, trace_id: str):
        """
        Returns top processes for a specific deeptrace.

        Args:
            device_id (str): The unique ID for the device.
            trace_id (str): The unique ID for the deeptrace.

        Returns:
            :obj:`Box`: The deeptrace top processes.

        Examples:
            Print top processes for a deeptrace.

            >>> top_processes = zdx.devices.get_deeptrace_top_processes('123456789', '987654321')
            ... print(top_processes)

        """
        return self._get(f"devices/{device_id}/deeptraces/{trace_id}/top-processes")



from restfly.endpoint import APIEndpoint

from pyzscaler.utils import convert_keys


class CloudAppsAPI(APIEndpoint):
    @staticmethod
    def _convert_ids_to_dict_list(id_list):
        """Helper function to convert a list of IDs into a list of dictionaries.

        Args:
            id_list (list): A list of IDs (str).

        Returns:
            list: A list of dictionaries, each with an 'id' key.
        """
        return [{"id": str(id)} for id in id_list]

    def bulk_update(self, sanction_state: str, **kwargs):
        """
        Updates application status and tag information for predefined or custom cloud applications based on the
        IDs specified.

        Args:
            sanction_state (str): The sanction state to apply to the cloud applications.

                Accepted values are:

                    ``sanctioned``: The cloud application is sanctioned.
                    ``unsanctioned``: The cloud application is unsanctioned.
                    ``any``: The cloud application is either sanctioned or unsanctioned.

            **kwargs:
                Optional keyword args

        Keyword Args:
            application_ids (list): A list of cloud application IDs to update.
            custom_tag_ids (list): A list of custom tag IDs to apply to the cloud applications.

        Returns:
            :obj:`dict`: The response from the ZIA API.

        Examples:
            Update the sanction state of a cloud application::

                zia.cloud_apps.bulk_update("sanctioned", application_ids=["12345"])

            Update the sanction state and custom tags of a cloud application::

                zia.cloud_apps.bulk_update("sanctioned", application_ids=["12345"], custom_tag_ids=["67890"])

        """
        # Mapping for user-friendly sanction state values to API-expected values
        sanction_state_mapping = {"sanctioned": "SANCTIONED", "unsanctioned": "UN_SANCTIONED", "any": "ANY"}

        # Convert user-friendly sanction state to ZIA API-expected value
        api_sanction_state = sanction_state_mapping.get(sanction_state.lower())
        if not api_sanction_state:
            raise ValueError(
                f"Invalid sanction state: {sanction_state}. Accepted values are 'sanctioned', 'unsanctioned', or 'any'."
            )

        payload = {"sanctionedState": api_sanction_state}

        # Process application_ids if provided in kwargs
        application_ids = kwargs.pop("application_ids", None)
        if application_ids is not None:
            payload["applicationIds"] = application_ids

        # Process custom_tag_ids if provided in kwargs
        custom_tag_ids = kwargs.pop("custom_tag_ids", None)
        if custom_tag_ids is not None:
            payload["customTags"] = self._convert_ids_to_dict_list(custom_tag_ids)

        return self._put("cloudApplications/bulkUpdate", json=payload).status_code

    def export_shadow_it_report(self, duration: str = "LAST_1_DAYS", **kwargs) -> str:
        """
        Export the Shadow IT Report (in CSV format) for the cloud applications recognized by Zscaler
        based on their usage in your organisation.

        Args:
            duration (str):
                Filters the data by using predefined time frames. Defaults to last day.

                Possible values: ``LAST_1_DAYS``, ``LAST_7_DAYS``, ``LAST_15_DAYS``, ``LAST_MONTH``, ``LAST_QUARTER``
            **kwargs:
                Arbitrary keyword arguments for filtering the report.

        Keyword Args:
            app_name (str): Filters the data based on the cloud application name that matches the specified string.
            order (dict):
                Sorts the list in increasing or decreasing order based on the specified attribute.

                Example format for this parameter:

                    ``order={"on": "RISK_SCORE", "by": "INCREASING"}``

                Possible values for ``on``:

                ``RISK_SCORE``, ``APPLICATION``, ``APPLICATION_CATEGORY``,
                ``SANCTIONED_STATE``, ``TOTAL_BYTES``, ``UPLOAD_BYTES``, ``DOWNLOAD_BYTES``, ``AUTHENTICATED_USERS``,
                ``TRANSACTION_COUNT``, ``UNAUTH_LOCATION``, ``LAST_ACCESSED``.

                Possible values for ``by``:

                ``INCREASING``, ``DECREASING``.
            application_category (str): Filters the data based on the cloud application category.

                Possible values: ``ANY``, ``NONE``, ``WEB_MAIL``, ``SOCIAL_NETWORKING``, ``STREAMING``, ``P2P``,
                ``INSTANT_MESSAGING``, ``WEB_SEARCH``, ``GENERAL_BROWSING``, ``ADMINISTRATION``, ``ENTERPRISE_COLLABORATION``,
                ``BUSINESS_PRODUCTIVITY``, ``SALES_AND_MARKETING``, ``SYSTEM_AND_DEVELOPMENT``, ``CONSUMER``, ``FILE_SHARE``,
                ``HOSTING_PROVIDER``, ``IT_SERVICES``, ``DNS_OVER_HTTPS``, ``HUMAN_RESOURCES``, ``LEGAL``, ``HEALTH_CARE``,
                ``FINANCE``, ``CUSTOM_CAPP``
            data_consumed (dict):
                Filters the data by cloud application usage in terms of total data uploaded and downloaded.

                Example format for this parameter:

                    data_consumed={"min": 100, "max": 1000}

                    ``min`` and ``max`` fields specify the range respectively.
            risk_index (int):
                Filters the data based on the risk index assigned to cloud applications.

                Possible values: ``1``, ``2``, ``3``, ``4``, ``5``
            sanctioned_state (str):
                Filters the data based on the status of cloud applications.

                Possible values: ``UN_SANCTIONED``, ``SANCTIONED``, ``ANY``
            employees (str):
                Filters the data based on the employee count of the cloud application vendor.

                Possible values: ``NONE``, ``RANGE_1_100``, ``RANGE_100_1000``, ``RANGE_1000_10000``,
                ``RANGE_10000_INF``
            supported_certifications (dict): Filters the cloud applications by security certifications.

                Example format for this parameter:

                    ``supported_certifications={"operation": "INCLUDE", "value": ["ISO_27001", "HIPAA"]}``

                Possible values for ``operation`` field: ``INCLUDE`` and ``EXCLUDE``.

                Possible values for ``value`` field: ``NONE``, ``CSA_STAR``, ``ISO_27001``, ``HIPAA``, ``FISMA``,
                ``FEDRAMP``, ``SOC2``, ``ISO_27018``, ``PCI_DSS``, ``ISO_27017``, ``SOC1``, ``SOC3``, ``GDPR``,
                ``CCPA``, ``FERPA``, ``COPPA``, ``HITECH``, ``EU_US_SWISS_PRIVACY_SHIELD``,
                ``EU_US_PRIVACY_SHIELD_FRAMEWORK``, ``CISP``, ``AICPA``, ``FIPS``, ``SAFE_BIOPHARMA``, ``ISAE_3000``,
                ``SSAE_18``, ``NIST``, ``ISO_14001``, ``SOC``, ``TRUSTE``, ``ISO_26262``, ``ISO_20252``, ``RGPD``,
                ``ISO_20243``, ``ISO_10002``, ``JIS_Q_15001_2017``, ``ISMAP``.
            source_ip_restriction (str):
                Filters the cloud applications based on whether they have source IP restrictions.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            mfa_support (str): Filters the cloud applications based on whether they support multi-factor authentication.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            admin_audit_logs (str): Filters the cloud applications based on whether they support admin audit logging.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            had_breach_in_last_3_years (str):
                Filters the cloud applications based on data breaches in the last three years.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            have_poor_items_of_service (str): Filters the cloud applications based on their terms of service.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            password_strength (str): Filters the cloud applications based on whether they require strong passwords.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            ssl_pinned (str): Filters the cloud applications based on whether they use SSL Pinning.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            evasive (str): Filters the cloud applications based on their capability to bypass traditional firewalls.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            have_http_security_header_support (str): Filters the cloud applications by the presence of security headers.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            dns_caa_policy (str): Filters the cloud applications by the presence of DNS CAA policy.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            have_weak_cipher_support (str): Filters the cloud applications based on the cryptographic keys used.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            ssl_certification_validity (str): Filters the cloud applications based on SSL certificate validity.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            file_sharing (str): Filters the cloud applications based on whether they include file-sharing provision.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            malware_scanning_content (str):
                Filters the cloud applications based on whether they include malware content.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            remote_access_screen_sharing (str):
                Filters the cloud applications based on whether they support remote access and screen sharing.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            sender_policy_framework (str):
                Filters the cloud applications based on whether they support Sender Policy Framework.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            domain_keys_identified_mail (str):
                Filters the cloud applications based on whether they support DomainKeys Identified Mail.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            domain_based_message_authentication (str):
                Filters the cloud applications based on whether they support Domain-based Message Authentication.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            vulnerable_disclosure_program (str):
                Filters the cloud applications based on whether they support Vulnerability Disclosure Policy.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            waf_support (str): Filters the cloud applications based on whether WAF is enabled for the applications.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            vulnerability (str):
                Filters the cloud applications based on whether they have published Common Vulnerabilities and
                Exposures (CVE).

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            valid_ssl_certificate (str):
                Filters the cloud applications based on whether they have a valid SSL certificate.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            data_encryption_in_transit (str):
                Filters the cloud applications based on whether they support data encryption in transit.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            vulnerable_to_heart_bleed (str):
                Filters the cloud applications based on whether they are vulnerable to Heartbleed attack.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            vulnerable_to_poodle (str):
                Filters the cloud applications based on whether they are vulnerable to Poodle attack.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            vulnerable_to_log_jam (str):
                Filters the cloud applications based on whether they are vulnerable to Logjam attack.

                Possible values: ``YES``, ``NO``, ``UNKNOWN``.
            cert_key_size (dict):
                Filters the data by the size of the SSL certificate public keys used by the cloud applications.

                Example format for this parameter:

                    ``cert_key_size={"operation": "INCLUDE", "value": ["BITS_2048", "BITS_256"]}``

                Possible values for ``operation`` field:

                ``INCLUDE``, ``EXCLUDE``.

                Possible values for ``value`` field:

                ``NONE``, ``UN_KNOWN``, ``BITS_2048``, ``BITS_256``,
                ``BITS_3072``, ``BITS_384``, ``BITS_4096``, ``BITS_1024``.

        Returns:
            :obj:`str`: The Shadow IT Report in CSV format.

        Examples:
            Export the Shadow IT Report for the last 7 days::

                report = zia.shadow_it.export_shadow_it_report('LAST_7_DAYS')

        Notes:
            Zscaler has a rate limit of 1 report per-minute, ensure you take this into account when calling this method.

        """
        payload = {"duration": duration}
        convert_keys(payload.update(kwargs))

        return self._post("shadowIT/applications/export", json=payload).text

    def export_shadow_it_csv(self, application: str, entity: str, duration: str = "LAST_1_DAYS", **kwargs):
        """
        Export the Shadow IT Report (in CSV format) for the list of users or known locations
        identified with using the cloud applications specified in the request. The report
        includes details such as user interactions, application category, application usage,
        number of transactions, last accessed time, etc.

        You can customize the report using various filters.

        Args:
            application (str): The cloud application for which user or location data must be retrieved.
                Note: Only one cloud application can be specified at a time.
            duration (str): Filters the data using predefined timeframes. Defaults to last day.

            Possible values: ``LAST_1_DAYS``, ``LAST_7_DAYS``, ``LAST_15_DAYS``, ``LAST_MONTH``, ``LAST_QUARTER``.
            entity (str): The entity type that the Shadow IT Report will be generated for.

            Possible values: ``USER``, ``LOCATION``.

        Keyword Args:
            order (dict): Sorts the list in increasing or decreasing order based on the specified attribute.

                Example format for this parameter:

                    ``order={"on": "RISK_SCORE", "by": "INCREASING"}``

                Possible values for ``on``:

                ``RISK_SCORE``, ``APPLICATION``, ``APPLICATION_CATEGORY``,
                ``SANCTIONED_STATE``, ``TOTAL_BYTES``, ``UPLOAD_BYTES``, ``DOWNLOAD_BYTES``, ``AUTHENTICATED_USERS``,
                ``TRANSACTION_COUNT``, ``UNAUTH_LOCATION``, ``LAST_ACCESSED``.

                Possible values for ``by``:

                ``INCREASING``, ``DECREASING``.
            download_bytes (dict): Filters by the amount of data (in bytes) downloaded from the application.
                ``min`` and ``max`` fields specify the range.
            upload_bytes (dict): Filters by the amount of data (in bytes) uploaded to the application.
                ``min`` and ``max`` fields specify the range.
            data_consumed (dict): Filters by the total amount of data uploaded and downloaded from the application.
                ``min`` and ``max`` fields specify the range.
            users (dict): Filters by user.
                ``id`` and ``name`` fields specify the user information.
            locations (dict): Filters by location.
                ``id`` and ``name`` fields specify the location information.
            departments (dict): Filters by department.
                ``id`` and ``name`` fields specify the department information.

        Returns:
            :obj:`str`: The Shadow IT Report in CSV format.

        Examples:
            Export the Shadow IT Report for GitHub the last 15 days::

                report = zia.shadow_it.export_shadow_it_report(application="Github", duration="LAST_15_DAYS")

        Notes:
            Zscaler has a rate limit of 1 report per-minute, ensure you take this into account when calling this method.
        """

        payload = {"application": application, "duration": duration}

        # Process user_ids, location_ids, and department_ids if provided in kwargs
        for key in ["users", "locations", "departments"]:
            id_list = kwargs.pop(key, None)
            if id_list is not None:
                payload[key] = self._convert_ids_to_dict_list(id_list)

        convert_keys(payload.update(kwargs))

        return self._post(f"shadowIT/applications/{entity}/exportCsv", json=payload).text

    def list_apps(self):
        """
        List all predefined and custom cloud applications by name and id.

        Returns:
            :obj:`BoxList` of :obj:`Box`: A list of cloud applications.

        Examples:
            List all cloud applications::

                apps = zia.cloud_apps.list_apps()
                for app in apps:
                    print(app.name)

        """
        return self._get("cloudApplications/lite")

    def list_custom_tags(self):
        """
        List all custom tags by name and id.

        Returns:
            :obj:`BoxList` of :obj:`Box`: A list of custom tags available to assign to cloud applications.

        Examples:
            List all custom tags::

                tags = zia.cloud_apps.list_custom_tags()
                for tag in tags:
                    print(tag.name)

        """
        return self._get("customTags")



import time

from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import chunker, convert_keys, snake_to_camel


class URLCategoriesAPI(APIEndpoint):
    def lookup(self, urls: list) -> BoxList:
        """
        Lookup the category for the provided URLs.

        Args:
            urls (list):
                The list of URLs to perform a category lookup on.

        Returns:
            :obj:`BoxList`: A list of URL category reports.

        Examples:
            >>> zia.url_categories.lookup(['example.com', 'test.com'])

        """

        # ZIA limits each API call to 100 URLs at a rate of 1 API call per second. pyZscaler simplifies this by allowing
        # users to submit any number of URLs and handle the chunking of the API calls on their behalf.
        if len(urls) > 100:
            results = BoxList()
            for chunk in chunker(urls, 100):
                results.extend(self._post("urlLookup", json=chunk))
                time.sleep(1)
            return results

        else:
            payload = urls
            return self._post("urlLookup", json=payload)

    def list_categories(self, custom_only: bool = False, only_counts: bool = False) -> BoxList:
        """
        Returns information on URL categories.

        Args:
            custom_only (bool):
                Returns only custom categories if True.
            only_counts (bool):
                Returns only URL and keyword counts if True.

        Returns:
            :obj:`BoxList`: A list of information for all or custom URL categories.

        Examples:
            List all URL categories:

            >>> zia.url_categories.list_categories()

            List only custom URL categories:

            >>> zia.url_categories.list_categories(custom_only=True)

        """
        payload = {
            "customOnly": custom_only,
            "includeOnlyUrlKeywordCounts": only_counts,
        }

        return self._get("urlCategories", params=payload)

    def get_quota(self) -> Box:
        """
        Returns information on URL category quota usage.

        Returns:
            :obj:`Box`: The URL quota statistics.

        Examples:
            >>> zia.url_categories.get_quota()

        """

        return self._get("urlCategories/urlQuota")

    def get_category(self, category_id: str) -> Box:
        """
        Returns URL category information for the provided category.

        Args:
            category_id (str):
                The unique identifier for the category (e.g. 'MUSIC')

        Returns:
            :obj:`Box`: The resource record for the category.

        Examples:
            >>> zia.url_categories.get_category('ALCOHOL_TOBACCO')

        """
        return self._get(f"urlCategories/{category_id}")

    def add_url_category(self, name: str, super_category: str, urls: list, **kwargs) -> Box:
        """
        Adds a new custom URL category.

        Args:
            name (str):
                Name of the URL category.
            super_category (str):
                The name of the parent category.
            urls (list):
                Custom URLs to add to a URL category.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            db_categorized_urls (list):
                URLs entered will be covered by policies that reference the parent category, in addition to this one.
            description (str):
                Description of the category.
            custom_category (bool):
                Set to true for custom URL category. Up to 48 custom URL categories can be added per organisation.
            ip_ranges (list):
                Custom IP addpress ranges associated to a URL category. This feature must be enabled on your tenancy.
            ip_ranges_retaining_parent_category (list):
                The retaining parent custom IP addess ranges associated to a URL category.
            keywords (list):
                Custom keywords associated to a URL category.
            keywords_retaining_parent_category (list):
                Retained custom keywords from the parent URL category that are associated with a URL category.

        Returns:
            :obj:`Box`: The newly configured custom URL category resource record.

        Examples:
            Add a new category for beers that don't taste good:

            >>> zia.url_categories.add_url_category(name='Beer',
            ...    super_category='ALCOHOL_TOBACCO',
            ...    urls=['xxxx.com.au', 'carltondraught.com.au'],
            ...    description="Beers that don't taste good.")

            Add a new category with IP ranges:

            >>> zia.url_categories.add_url_category(name='Beer',
            ...    super_category='FINANCE',
            ...    urls=['finance.google.com'],
            ...    description="Google Finance.",
            ...    ip_ranges=['10.0.0.0/24'])

        """

        payload = {
            "type": "URL_CATEGORY",
            "superCategory": super_category,
            "configuredName": name,
            "urls": urls,
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post("urlCategories", json=payload)

    def add_tld_category(self, name: str, tlds: list, **kwargs) -> Box:
        """
        Adds a new custom TLD category.

        Args:
            name (str):
                The name of the TLD category.
            tlds (list):
                A list of TLDs in the format '.tld'.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            description (str):
                Description of the category.

        Returns:
            :obj:`Box`: The newly configured custom TLD category resource record.

        Examples:
            Create a category for all 'developer' sites:

            >>> zia.url_categories.add_tld_category(name='Developer Sites',
            ...    urls=['.dev'],
            ...    description="Sites that are likely run by developers.")

        """

        payload = {
            "type": "TLD_CATEGORY",
            "superCategory": "USER_DEFINED",  # TLDs can only be added in USER_DEFINED category
            "configuredName": name,
            "urls": tlds,  # ZIA API reuses the 'urls' key for tlds
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post("urlCategories", json=payload)

    def update_url_category(self, category_id: str, **kwargs) -> Box:
        """
        Updates a URL category.

        Args:
            category_id (str):
                The unique identifier of the URL category.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            name (str):
                The name of the URL category.
            urls (list):
                Custom URLs to add to a URL category.
            db_categorized_urls (list):
                URLs entered will be covered by policies that reference the parent category, in addition to this one.
            description (str):
                Description of the category.
            ip_ranges (list):
                Custom IP addpress ranges associated to a URL category. This feature must be enabled on your tenancy.
            ip_ranges_retaining_parent_category (list):
                The retaining parent custom IP addess ranges associated to a URL category.
            keywords (list):
                Custom keywords associated to a URL category.
            keywords_retaining_parent_category (list):
                Retained custom keywords from the parent URL category that are associated with a URL category.

        Returns:
            :obj:`Box`: The updated URL category resource record.

        Examples:
            Update the name of a category:

            >>> zia.url_categories.update_url_category('CUSTOM_01',
            ...    name="Wines that don't taste good.")

            Update the urls of a category:

            >>> zia.url_categories.update_url_category('CUSTOM_01',
            ...    urls=['www.yellowtailwine.com'])

        """

        payload = convert_keys(self.get_category(category_id))

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._put(f"urlCategories/{category_id}", json=payload)

    def add_urls_to_category(self, category_id: str, urls: list) -> Box:
        """
        Adds URLS to a URL category.

        Args:
            category_id (str):
                The unique identifier of the URL category.
            urls (list):
                Custom URLs to add to a URL category.

        Returns:
            :obj:`Box`: The updated URL category resource record.

        Examples:
            >>> zia.url_categories.add_urls_to_category('CUSTOM_01',
            ...    urls=['example.com'])

        """

        payload = convert_keys(self.get_category(category_id))
        payload["urls"] = urls

        return self._put(f"urlCategories/{category_id}?action=ADD_TO_LIST", json=payload)

    def delete_urls_from_category(self, category_id: str, urls: list) -> Box:
        """
        Deletes URLS from a URL category.

        Args:
            category_id (str):
                The unique identifier of the URL category.
            urls (list):
                Custom URLs to delete from a URL category.

        Returns:
            :obj:`Box`: The updated URL category resource record.

        Examples:
            >>> zia.url_categories.delete_urls_from_category('CUSTOM_01',
            ...    urls=['example.com'])

        """
        current_config = self.get_category(category_id)
        payload = {"configuredName": current_config["configured_name"], "urls": urls}  # Required for successful call

        return self._put(f"urlCategories/{category_id}?action=REMOVE_FROM_LIST", json=payload)

    def delete_from_category(self, category_id: str, **kwargs):
        """
        Deletes the specified items from a URL category.

        Args:
            category_id (str):
                The unique id for the URL category.
            **kwargs:
                Optional parameters.

        Keyword Args:
            keywords (list):
                A list of keywords that will be deleted.
            keywords_retaining_parent_category (list):
                A list of keywords retaining their parent category that will be deleted.
            urls (list):
                A list of URLs that will be deleted.
            db_categorized_urls (list):
                A list of URLs retaining their parent category that will be deleted

        Returns:
            :obj:`Box`: The updated URL category resource record.

        Examples:
            Delete URLs retaining parent category from a custom category:

            >>> zia.url_categories.delete_from_category(
            ...    category_id="CUSTOM_01",
            ...    db_categorized_urls=['twitter.com'])

            Delete URLs and URLs retaining parent category from a custom category:

            >>> zia.url_categories.delete_from_category(
            ...    category_id="CUSTOM_01",
            ...    urls=['news.com', 'cnn.com'],
            ...    db_categorized_urls=['google.com, bing.com'])

        """
        current_config = self.get_category(category_id)

        payload = {
            "configured_name": current_config["configured_name"],  # Required for successful call
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[key] = value

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._put(f"urlCategories/{category_id}?action=REMOVE_FROM_LIST", json=payload)

    def delete_category(self, category_id: str) -> int:
        """
        Deletes the specified URL category.

        Args:
            category_id (str):
                The unique identifier for the category.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            >>> zia.url_categories.delete_category('CUSTOM_01')

        """

        return self._delete(f"urlCategories/{category_id}", box=False).status_code



import os

from box import Box
from restfly.session import APISession

from pyzscaler import __version__
from pyzscaler.zdx.admin import AdminAPI
from pyzscaler.zdx.apps import AppsAPI
from pyzscaler.zdx.devices import DevicesAPI
from pyzscaler.zdx.session import SessionAPI
from pyzscaler.zdx.users import UsersAPI


class ZDX(APISession):
    """
    A Controller to access Endpoints in the Zscaler Digital Experience (ZDX) API.

    The ZDX object stores the session token and simplifies access to CRUD options within the ZDX Portal.

    Attributes:
        client_id (str): The ZDX Client ID generated from the ZDX Portal.
        client_secret (str): The ZDX Client Secret generated from the ZDX Portal.
        cloud (str): The Zscaler cloud for your tenancy, accepted values are below. Defaults to ``zdxcloud``.

            * ``zdxcloud``
            * ``zdxbeta``

        override_url (str):
            If supplied, this attribute can be used to override the production URL that is derived
            from supplying the `cloud` attribute. Use this attribute if you have a non-standard tenant URL
            (e.g. internal test instance etc). When using this attribute, there is no need to supply the `cloud`
            attribute. The override URL will be prepended to the API endpoint suffixes. The protocol must be included
            i.e. http:// or https://.

    """

    _vendor = "Zscaler"
    _product = "pyZscaler"
    _backoff = 3
    _build = __version__
    _box = True
    _box_attrs = {"camel_killer_box": True}
    _env_base = "ZDX"
    _env_cloud = "zdxcloud"
    _url = "https://api.zdxcloud.net/v1"

    def __init__(self, **kw):
        self._client_id = kw.get("client_id", os.getenv(f"{self._env_base}_CLIENT_ID"))
        self._client_secret = kw.get("client_secret", os.getenv(f"{self._env_base}_CLIENT_SECRET"))
        self._cloud = kw.get("cloud", os.getenv(f"{self._env_base}_CLOUD", self._env_cloud))
        self._url = kw.get("override_url", os.getenv(f"{self._env_base}_OVERRIDE_URL")) or f"https://api.{self._cloud}.net/v1"
        self.conv_box = True
        super(ZDX, self).__init__(**kw)

    def _build_session(self, **kwargs) -> Box:
        """Creates a ZCC API session."""
        super(ZDX, self)._build_session(**kwargs)
        self._auth_token = self.session.create_token(client_id=self._client_id, client_secret=self._client_secret).token
        return self._session.headers.update({"Authorization": f"Bearer {self._auth_token}"})

    @property
    def session(self):
        """The interface object for the :ref:`ZDX Session interface <zdx-session>`."""
        return SessionAPI(self)

    @property
    def admin(self):
        """The interface object for the :ref:`ZDX Admin interface <zdx-admin>`."""
        return AdminAPI(self)

    @property
    def apps(self):
        """The interface object for the :ref:`ZDX Apps interface <zdx-apps>`."""
        return AppsAPI(self)

    @property
    def devices(self):
        """The interface object for the :ref:`ZDX Devices interface <zdx-devices>`."""
        return DevicesAPI(self)

    @property
    def users(self):
        """The interface object for the :ref:`ZDX Users interface <zdx-users>`."""
        return UsersAPI(self)
